#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import math, Numeric 

x = Numeric.arange( 0, 3*math.pi, math.pi/10 )
y = Numeric.sin(x)

a = biggles.FramedArray( 2, 2, title='title' )
a.aspect_ratio = 0.75
a.xlabel = "x label"
a.ylabel = "y label"
a.uniform_limits = 1
a.cellspacing = 1.

a.add( biggles.LineY(0, type='dot') )

a[0,0].add( biggles.Curve(x, .25*y) )
a[0,1].add( biggles.Curve(x, .50*y) )
a[1,0].add( biggles.Curve(x, .75*y) )
a[1,1].add( biggles.Curve(x, y) )

#a.write_img( 400, 400, "example5.png" )
#a.write_eps( "example5.eps" )
a.show()



#
# $Id: __init__.py,v 1.79 2004/03/08 23:40:25 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

__version__ = '1.6.4'

from biggles import		 \
	Circle			,\
	Circles			,\
	ColoredPoint		,\
	ColoredPoints		,\
	Curve			,\
	DataArc			,\
	DataBox			,\
	DataInset		,\
	DataLabel		,\
	DataLine		,\
	Density			,\
	Ellipse			,\
	Ellipses		,\
	ErrorBarsX		,\
	ErrorBarsY		,\
	FillAbove		,\
	FillBelow		,\
	FillBetween		,\
	FramedArray		,\
	FramedPlot		,\
	Geodesic		,\
	Histogram		,\
	LineX			,\
	LineY			,\
	LowerLimits		,\
	OldCustomFramedPlot	,\
	OldKey			,\
	Plot			,\
	PlotArc			,\
	PlotBox			,\
	PlotInset		,\
	PlotKey			,\
	PlotLabel		,\
	PlotLine		,\
	Point			,\
	Points			,\
	Slope			,\
	SymmetricErrorBarsX	,\
	SymmetricErrorBarsY	,\
	Table			,\
	Text			,\
	UpperLimits		,\
	multipage

from config import		 \
	configure

from contour import		 \
	Contour			,\
	Contours

from func import		 \
	plot			,\
	read_column		,\
	read_matrix		,\
	read_rows

from hammer import		 \
	HammerAitoffPlot

# aliases
Arc = DataArc
Box = DataBox
Inset = PlotInset
Label = DataLabel
Line = DataLine
SymmetricXErrorBars = SymmetricErrorBarsX
SymmetricYErrorBars = SymmetricErrorBarsY
XErrorBars = ErrorBarsX
YErrorBars = ErrorBarsY

class _deprecated:

	def __init__( self, obj, old, new, harrass=1 ):
		self.obj = obj
		self.old = old
		self.new = new
		self.harrass = harrass

	def __call__( self, *args, **kw ):
		import sys
		if self.harrass == 1:
			sys.stderr.write( \
				"biggles: %s is deprecated - use %s instead\n" \
				% (self.old, self.new) )
		return apply( self.obj, args, kw )

# XXX:deprecated 1.0
Plot2D = _deprecated( FramedPlot, "Plot2D", "FramedPlot" )

# XXX:deprecated 1.3
ErrorEllipses = _deprecated( Ellipses, "ErrorEllipses", "Ellipses"  )
LabelData = _deprecated( DataLabel, "LabelData", "DataLabel" )
LabelPlot = _deprecated( PlotLabel, "LabelPlot", "PlotLabel" )
LineKey = _deprecated( OldKey, "LineKey", "PlotKey" )
LineSlope = _deprecated( Slope, "LineSlope", "Slope" )
SymbolKey = _deprecated( OldKey, "SymbolKey", "PlotKey" )

# XXX:deprecated 1.5
CustomFramedPlot = _deprecated( \
	OldCustomFramedPlot, "CustomFramedPlot", "FramedPlot", 0 )
readcolumn = _deprecated( read_column, "readcolumn", "read_column", 0 )

try:
	del _biggles
	del biggles
	del config
	del contour
	del func
	del geometry
	del hammer
except NameError:
	pass




#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import Numeric

#
# Create example 2-dimensional data set of two solitons colliding.
#
n = 64
x = Numeric.arange( -10., 10., 20./n )
t = Numeric.arange( -1., 1., 2./n )
z = Numeric.zeros( (len(x),len(t)), Numeric.Float )

for i in range(len(x)):
	for j in range(len(t)):
		z[i,j] = -12. * (3. + 4.*Numeric.cosh(2.*x[i]-8.*t[j]) \
			+ Numeric.cosh(4.*x[i] - 64.*t[j])) / \
			(3.*Numeric.cosh(x[i]-28.*t[j]) \
			+ Numeric.cosh(3.*x[i]-36.*t[j]))**2

#
# Make contour component.
#
c = biggles.Contours( z, x, t, color="red" )

#
# For fine-grained color control, the Contours component allows you to
# specify a function which returns the color applied to each contour line.
# The arguments passed to the function are:
#
#	i	integer index of contour (0,..,n-1)
#	n 	total number of contours
#	z0	z value of contour
#	z_min	minimum z contour value
#	z_max	maximum z contour value
#
# The function should return a valid color, or None for the default.
#
# Here we show how to set every other contour to blue. The remaining 
# contours are drawn with the default color, defined above to be red.
#
def even_blue( i, n, z0, z_min, z_max ):
	if i % 2 == 0:
		return 0x0000ff
	return None

c.func_color = even_blue

#
# Similarly, Contours accepts similar functions for line type (.func_linestyle)
# and width (.func_linewidth). The arguments passed are the same.
#

#
# Make framed plot container and add contour component.
#
p = biggles.FramedPlot()
p.add( c )

#
# Output.
#
#p.write_img( 400, 400, "example8.png" )
#p.write_eps( "example8.eps" )
p.show()



## Automatically adapted for numpy.oldnumeric Apr 04, 2007 by alter_code1.py

#
# $Id: contour.py,v 1.21 2004/03/08 04:30:06 mrnolta Exp $
#
# Copyright (C) 2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

from biggles import \
	_series, _message, \
	_LineComponent,  _PathObject, _PlotComponent, BigglesError
from geometry import *
import _biggles

import numpy.oldnumeric as Numeric

def _span( a, b, n ):
	return a + float(b - a)*Numeric.arange( 0, n, 1, Numeric.Float )/(n-1)

def _pop2( x, i, j ):
	if i < j:
		b = x.pop( j )
		a = x.pop( i )
	elif i > j:
		a = x.pop( i )
		b = x.pop( j )
	return a, b

def _unzip( line ):
	x = []
	y = []
	for x0,y0 in line:
		x.append( x0 )
		y.append( y0 )
	return x, y

class Contour( _LineComponent ):

	def __init__( self, x, y, z, z0, **kw ):
		_LineComponent.__init__( self )
		self.kw_init( kw )
		self.x = x
		self.y = y
		self.z = z
		self.z0 = z0

	def limits( self ):
		p = min(self.x), min(self.y)
		q = max(self.x), max(self.y)
		return BoundingBox( p, q )

	def _get_contours( self ):
		segs = _biggles.contour_segments( \
			self.x, self.y, self.z, self.z0 )
		open = [[ segs[0][0], segs[0][1] ]]
		closed = []
		for a,b in segs[1:]:
			xxx = []
			for i in range(len(open)):
				begin = open[i][0]
				end = open[i][-1]
				if a == begin:
					xxx.append( (i,0,b) )
				elif a == end:
					xxx.append( (i,1,b) )
				if b == begin:
					xxx.append( (i,0,a) )
				elif b == end:
					xxx.append( (i,1,a) )
			if len(xxx) == 0:
				open.append( [a,b] )
			elif len(xxx) == 1:
				i,end,pt = xxx[0]
				if end == 0:
					open[i].insert( 0, pt )
				else:
					open[i].append( pt )
			elif len(xxx) == 2:
				i0,end0,pt0 = xxx[0]
				i1,end1,pt1 = xxx[1]
				if i0 == i1:
					# closed
					l0 = open.pop( i0 )
					l0.append( l0[0] )
					closed.append( l0 )
				else:
					l0, l1 = _pop2( open, i0, i1 )
					m = None
					if end0==1 and end1==0:
						m = l0 + l1
					elif  end0==0 and end1==1:
						m = l1 + l0
					elif end0==0 and end1==0:
						l0.reverse()
						m = l0 + l1
					elif end0==1 and end1==1:
						l1.reverse()
						m = l0 + l1
					if m is not None:
					 	open.append( m )
					else:
						_message( "contour: m is None" )
			elif len(xxx) > 2:
				_message( "contour: len(xxx) > 2" )
		return open + closed

	def make( self, context ):
		lines = self._get_contours()
		for line in lines:
			x, y = _unzip( line )
			u, v = context.geom.call_vec( x, y )
			self.add( _PathObject(u, v) )

def _func_color_black( i, n, z0, z_min, z_max ):
	return 0x000000

def _func_linetype_dotneg( i, n, z0, z_min, z_max ):
	if z0 < 0:
		return "dotted"
	return "solid"

def _func_linewidth_placeholder( i, n, z0, z_min, z_max ):
	return 1

class Contours( _PlotComponent ):

	_named_func_color = {
		"black"			: _func_color_black,
	}

	_named_func_linetype = {
		"dotted-negative"	: _func_linetype_dotneg,
	}

	_named_func_linewidth = {
		"placeholder"		: _func_linewidth_placeholder,
	}

	def __init__( self, z, x=None, y=None, zrange=None, **kw ):
		_PlotComponent.__init__( self )
		#apply( self.conf_setattr, ("Contours",), kw )
		self.conf_setattr( "Contours" )
		self.kw_init( kw )
		self.z = z
		self.x = x
		self.y = y
		self.zrange = zrange

	def _get_coords( self ):
		dim = self.z.shape
		x = self.x
		if x is None:
			x = range(dim[0])
		y = self.y
		if y is None:
			y = range(dim[1])
		return x, y, self.z

	def limits( self ):
		x, y, z = self._get_coords()
		return BoundingBox( (min(x),min(y)), (max(x),max(y)) )

	def make( self, context ):
		self.clear()

		x, y, z = self._get_coords()
		limits = self.limits()
		xr = limits.xrange()
		yr = limits.yrange()

		zr = self.zrange
		if zr is None:
			zr = _biggles.range( z )

		levels = self.levels
		if type(levels) == type(0):
			levels = _series( 1, self.levels, \
				float(zr[1]-zr[0])/(self.levels+1), zr[0] )

		colorfunc = self.func_color
		if type(colorfunc) == type(""):
			colorfunc = self._named_func_color[colorfunc]

		linefunc = self.func_linetype
		if type(linefunc) == type(""):
			linefunc = self._named_func_linetype[linefunc]

		widthfunc = self.func_linewidth
		if type(widthfunc) == type(""):
			widthfunc = self._named_func_linewidth[widthfunc]

		nlevels = len(levels)
		for i in range(nlevels):
			kw = {}
			z0 = levels[i]
			args = i, nlevels, z0, zr[0], zr[1]
			if colorfunc is not None:
				color = apply( colorfunc, args )
				if color is not None:
					kw["color"] = color
			if linefunc is not None:
				linetype = apply( linefunc, args )
				if linetype is not None:
					kw["linetype"] = linetype
			if widthfunc is not None:
				linewidth = apply( widthfunc, args )
				if linewidth is not None:
					kw["linewidth"] = linewidth
			c = apply( Contour, (x, y, z, z0), kw )
			self.add( c )




#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
from Numeric import *

def mag( vec ):
	return sqrt(sum(vec*vec, -1))

def make_coloredpoints_plot():

	# This is the magic recipe for an array of points from (0,0) to (10,10)
	(x,y) = reshape(indices([10+1,10+1]), (2,-1))

	# Let's color the points by their distance from the point (3,7)
	center    = (3,7)
	rad       = mag( transpose([x,y]) - center )
	scaledrad = (1 - rad/max(rad))[:,NewAxis]

	# Go from light blue to intense red.
	minColor = array([0.6, 0.9, 1.0])
	maxColor = array([1.0, 0.2, 0.2])
	colorrad = minColor + scaledrad*(maxColor-minColor)

	cp = biggles.ColoredPoints(x, y, colorrad, \
					type='filled circle', size=6 )

	# make plot
	p = biggles.FramedPlot()
	p.title = "Colored Points Plot"
	p.add( cp )

	return p

def make_density_plot():

	a = reshape(arange(90.0), (5,6,3))
	a[...,1] = 100 - a[...,1]

	d = biggles.Density( 1-(a/max(a.flat)), [[0,0], [5,10]] )

	# make plot
	p = biggles.FramedPlot()
	p.title = "Density"
	p.add( d )

	return p

p1 = make_coloredpoints_plot()
p2 = make_density_plot()

t = biggles.Table( 1, 2 )
t.aspect_ratio = 0.5
t[0,0] = p1
t[0,1] = p2

#t.write_img( 400, 400, "example9.png" )
#t.write_eps( "example9.eps" )
t.show()



## Automatically adapted for numpy.oldnumeric Apr 04, 2007 by alter_code1.py

#
# $Id: biggles.py,v 1.232 2003/05/12 22:14:41 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import copy, math, os, string
import numpy.oldnumeric as Numeric

import config, _biggles
from geometry import *

import libplot.renderer
renderer = libplot.renderer

# miscellaneous ---------------------------------------------------------------

_true, _false = 1, 0

def _floor(x):
	return long(math.floor(x))

def _ceil(x):
	return long(math.ceil(x))

def _tsil(x):
	l = list(x)
	l.reverse()
	return l

def _message( s ):
	print "biggles:", s 

def _series( m, n, a=1, b=0 ):
	return map( lambda x,y=a,z=b: x*y+z, range(m, n+1) )

def _first_not_none( *args ):
	for arg in args:
		if arg is not None:
			return arg
	return None

class _Alias:

	def __init__( self, *args ):
		self.__dict__["objs"] = args[:]

	def __call__( self, *args, **kw ):
		for obj in self.objs:
			apply( obj, args, kw )

	def __getattr__( self, name ):
		objs = []
		for obj in self.objs:
			objs.append( getattr(obj, name) )
		return apply( _Alias, objs )

	def __setattr__( self, name, value ):
		for obj in self.objs:
			setattr( obj, name, value )

	def __setitem__( self, key, value ):
		for obj in self.objs:
			obj[key] = value

# BigglesError ----------------------------------------------------------------

class BigglesError( Exception ):
	pass

# relative size ---------------------------------------------------------------

def _size_relative( relsize, bbox ):
	w = bbox.width()
	h = bbox.height()
	yardstick = math.sqrt(8) * w * h / (w + h)
	return (relsize/100.) * yardstick

def _fontsize_relative( relsize, bbox, device ):
	devsize = _size_relative( relsize, bbox )
	devsize_min = _size_relative(
		config.value('default','fontsize_min'), device.bbox )
	return max( devsize, devsize_min )

# _PlotContext -----------------------------------------------------------------

class _PlotGeometry:

	_logfunc = math.log10
	_logfunc_vec = Numeric.log10

	def __init__( self, src, dest, xlog=0, ylog=0 ):
		self.src_bbox = src
		self.dest_bbox = dest
		self.xlog = xlog
		self.ylog = ylog
		a, b = src.lowerleft()
		c, d = src.upperright()
		if xlog:
			a = self._logfunc(a)
			c = self._logfunc(c)
		if ylog:
			b = self._logfunc(b)
			d = self._logfunc(d)
		fsrc = BoundingBox( (a,b), (c,d) )
		self.aff = RectilinearMap( fsrc, dest )

	def __call__( self, x, y ):
		u, v = x, y
		if self.xlog: u = self._logfunc(x)
		if self.ylog: v = self._logfunc(y)
		return self.aff( u, v )

	def call_vec( self, x, y ):
		u = Numeric.asarray( x )
		v = Numeric.asarray( y )
		if self.xlog: u = self._logfunc_vec(u)
		if self.ylog: v = self._logfunc_vec(v)
		return self.aff.call_vec( u, v )

	def geodesic( self, x, y, div=1 ):
		return [(x, y)]

class _PlotContext:

	def __init__( self, device, dev, data, xlog=0, ylog=0 ):
		self.draw = device
		self.dev_bbox = dev
		self.data_bbox = data
		self.xlog = xlog
		self.ylog = ylog
		self.geom = _PlotGeometry( data, dev, xlog=xlog, ylog=ylog )
		self.plot_geom = _PlotGeometry( BoundingBox((0,0),(1,1)), dev )

	def do_clip( self ):
		xr = self.dev_bbox.xrange()
		yr = self.dev_bbox.yrange()
		self.draw.set( "cliprect", (xr[0], xr[1], yr[0], yr[1]) )

# _StyleKeywords --------------------------------------------------------------

def _kw_func_relative_fontsize( context, key, value ):
	device_size = _fontsize_relative( value, context.dev_bbox, context.draw )
	context.draw.set( key, device_size )

def _kw_func_relative_size( context, key, value ):
	device_size = _size_relative( value, context.dev_bbox )
	context.draw.set( key, device_size )

def _kw_func_relative_width( context, key, value ):
	device_width = _size_relative( value/10., context.dev_bbox )
	context.draw.set( key, device_width )

class _StyleKeywords:

	kw_style = None
	kw_defaults = {}
	kw_rename = {}
	kw_func = {
		'fontsize' : _kw_func_relative_fontsize,
		'linewidth' : _kw_func_relative_width,
		'symbolsize' : _kw_func_relative_size,
	}

	def kw_init( self, kw=None ):
		self.kw_style = {}
		self.kw_style.update( self.kw_defaults )
		if kw is not None:
			for key, value in kw.items():
				self.kw_set( key, value )

	def kw_set( self, key, value ):
		if self.kw_style is None:
			self.kw_init()
		key = self.kw_rename.get( key, key )
		self.kw_style[key] = value

	def style( self, **kw ):
		for key,val in kw.items():
			self.kw_set( key, val )

	def kw_get( self, key, notfound=None ):
		if self.kw_style is not None:
			return self.kw_style.get( key, notfound )
		else:
			return None

	def kw_predraw( self, context ):
		context.draw.save_state()
		if self.kw_style is not None:
			for key, value in self.kw_style.items():
				if self.kw_func.has_key(key):
					method = self.kw_func[key]
					apply( method, (context,key,value) )
				else:
					context.draw.set( key, value )

	def kw_postdraw( self, context ):
		context.draw.restore_state()

# _ConfAttributes -----------------------------------------------------------

class _ConfAttributes:

	def conf_setattr( self, section, **kw ):
		import copy, string
		sec = config.options( section )
		if sec is not None:
			for key,val in sec.items():
				x = string.split( key, "." )
				obj = self
				for y in x[:-1]:
					obj = getattr( obj, y )
				setattr( obj, x[-1], copy.copy(val) )
		for key,val in kw.items():
			setattr( self, key, copy.copy(val) )

# _DeviceObject ---------------------------------------------------------------

class _DeviceObject( _StyleKeywords ):

	def bbox( self, context ):
		return BoundingBox()

	def draw( self, context ):
		raise BigglesError

	def render( self, context ):
		self.kw_predraw( context )
		self.draw( context )
		self.kw_postdraw( context )

class _SymbolObject( _DeviceObject ):

	kw_rename = {
		'type' : 'symboltype',
		'size' : 'symbolsize',
	}

	def __init__( self, pos, **kw ):
		self.kw_init( kw )
		self.pos = pos

	def bbox( self, context ):
		self.kw_predraw( context )
		symbolsize = context.draw.get( 'symbolsize' )
		self.kw_postdraw( context )

		dp = symbolsize/2, symbolsize/2
		p = pt_sub( self.pos, dp )
		q = pt_add( self.pos, dp )
		return BoundingBox( p, q )

	def draw( self, context ):
		context.draw.symbol( self.pos )

class _TextObject( _DeviceObject ):

	kw_defaults = {
		'textangle'	: 0,
		'texthalign'	: 'center',
		'textvalign'	: 'center',
	}

	kw_rename = {
		'face'		: 'fontface',
		'size'		: 'fontsize',
		'angle'		: 'textangle',
		'halign'	: 'texthalign',
		'valign'	: 'textvalign',
	}

	def __init__( self, pos, str, **kw ):
		self.kw_init( kw )
		self.pos = pos
		self.str = str

	__halign_offset = { 'right':(-1,0), 'center':(-.5,.5), 'left':(0,1) }
	__valign_offset = { 'top':(-1,0), 'center':(-.5,.5), 'bottom':(0,1) }

	def bbox( self, context ):
		self.kw_predraw( context )
		angle = context.draw.get( 'textangle' ) * math.pi/180.
		halign = context.draw.get( 'texthalign' )
		valign = context.draw.get( 'textvalign' )
		width = context.draw.textwidth( self.str )
		height = context.draw.textheight( self.str )
		self.kw_postdraw( context )

		hvec = pt_mul( width, _TextObject.__halign_offset[halign] )
		vvec = pt_mul( height, _TextObject.__valign_offset[valign] )

		p = self.pos[0] + hvec[0], self.pos[1] + vvec[0]
		q = self.pos[0] + hvec[1], self.pos[1] + vvec[1]

		bb = BoundingBox( p, q )
		bb.rotate( angle, self.pos )
		return bb

	def draw( self, context ):
		#bb = self.bbox( context )
		#context.draw.rect( bb.lowerleft(), bb.upperright() )
		context.draw.text( self.pos, self.str )

class _LabelsObject( _DeviceObject ):

	kw_defaults = {
		'textangle'	: 0,
		'texthalign'	: 'center',
		'textvalign'	: 'center',
	}

	kw_rename = {
		'face'		: 'fontface',
		'size'		: 'fontsize',
		'angle'		: 'textangle',
		'halign'	: 'texthalign',
		'valign'	: 'textvalign',
	}

	def __init__( self, points, labels, **kw ):
		self.kw_init( kw )
		self.points = points
		self.labels = labels

	__halign_offset = { 'right':(-1,0), 'center':(-.5,.5), 'left':(0,1) }
	__valign_offset = { 'top':(-1,0), 'center':(-.5,.5), 'bottom':(0,1) }

	def bbox( self, context ):
		bb = BoundingBox()
		self.kw_predraw( context )

		angle = context.draw.get( 'textangle' ) * math.pi/180.
		halign = context.draw.get( 'texthalign' )
		valign = context.draw.get( 'textvalign' )

		height = context.draw.textheight( self.labels[0] )
		ho = _LabelsObject.__halign_offset[halign]
		vo = _LabelsObject.__valign_offset[valign]

		for i in range(len(self.labels)):
			pos = self.points[i]
			width = context.draw.textwidth( self.labels[i] )

			p = pos[0] + width * ho[0], pos[1] + height * vo[0]
			q = pos[0] + width * ho[1], pos[1] + height * vo[1]

			bb_label = BoundingBox( p, q )
			if angle != 0:
				bb_label.rotate( angle, pos )
			bb.union( bb_label )

		self.kw_postdraw( context )
		return bb

	def draw( self, context ):
		for i in range(len(self.labels)):
			context.draw.text( self.points[i], self.labels[i] )

class _LineTextObject( _TextObject ):

	kw_rename = {
		'face'		: 'fontface',
		'size'		: 'fontsize',
	}

	def __init__( self, p, q, str, offset, **kw ):
		self.kw_init( kw )
		self.str = str

		midpoint = pt_mul( 0.5, pt_add(p, q) )
		direction = pt_unit( pt_sub(q, p) )
		angle = pt_angle( direction )
		direction = pt_rot( direction, math.pi/2 )
		self.pos = pt_add( midpoint, pt_mul(offset, direction) )

		self.kw_set( 'textangle', angle * 180./math.pi )
		self.kw_set( 'texthalign', 'center' )
		if offset > 0:
			self.kw_set( 'textvalign', 'bottom' )
		else:
			self.kw_set( 'textvalign', 'top' )

class _LineObject( _DeviceObject ):

	kw_rename = {
		'width'		: 'linewidth',
		'type'		: 'linetype',
	}

	def __init__( self, p, q, **kw ):
		self.kw_init( kw )
		self.p = p
		self.q = q

	def bbox( self, context ):
		return BoundingBox( self.p, self.q )

	def draw( self, context ):
		context.draw.line( self.p, self.q )

class _PolygonObject( _DeviceObject ):

	kw_rename = {
		'width'		: 'linewidth',
		'type'		: 'linetype',
	}

	def __init__( self, points, **kw ):
		self.kw_init( kw )
		self.points = points

	def bbox( self, context ):
		return apply( BoundingBox, self.points )

	def draw( self, context ):
		context.draw.polygon( self.points )

class _PathObject( _DeviceObject ):

	kw_rename = {
		'width'		: 'linewidth',
		'type'		: 'linetype',
	}

	def __init__( self, x, y, **kw ):
		self.kw_init( kw )
		self.x = x
		self.y = y

	def bbox( self, context ):
		xmin, xmax = _biggles.range( self.x )
		ymin, ymax = _biggles.range( self.y )
		return BoundingBox( (xmin,ymin), (xmax,ymax) )

	def draw( self, context ):
		context.draw.curve( self.x, self.y )

class _SymbolsObject( _DeviceObject ):

	kw_rename = {
		'type' : 'symboltype',
		'size' : 'symbolsize',
	}

	def __init__( self, x, y, **kw ):
		self.kw_init( kw )
		self.x = x
		self.y = y

	def bbox( self, context ):
		xmin, xmax = _biggles.range( self.x )
		ymin, ymax = _biggles.range( self.y )
		return BoundingBox( (xmin,ymin), (xmax,ymax) )

	def draw( self, context ):
		context.draw.symbols( self.x, self.y )

class _ColoredSymbolsObject( _DeviceObject ):

	kw_rename = {
		'type' : 'symboltype',
		'size' : 'symbolsize',
	}

	def __init__( self, x, y, c, **kw ):
		self.kw_init( kw )
		self.x = x
		self.y = y
		self.c = c

	def bbox( self, context ):
		xmin, xmax = _biggles.range( self.x )
		ymin, ymax = _biggles.range( self.y )
		return BoundingBox( (xmin,ymin), (xmax,ymax) )

	def draw( self, context ):
		context.draw.colored_symbols( self.x, self.y, self.c )

class _DensityObject( _DeviceObject ):

	kw_rename = {
	}

	def __init__( self, densgrid, ((xmin,ymin), (xmax,ymax)), **kw ):
		self.kw_init( kw )
		self.densgrid = densgrid
		self.extent   = ( (xmin,ymin), (xmax,ymax) )

	def bbox( self, context ):
		return apply( BoundingBox, self.extent )

	def draw( self, context ):
		#from numpy.oldnumeric import rank
		#if rank(self.densgrid) == 3:
		if len(self.densgrid.shape) == 3:
			context.draw.color_density_plot( self.densgrid, self.extent )
		else:
			context.draw.density_plot( self.densgrid, self.extent )

class _EllipseObject( _DeviceObject ):

	def __init__( self, p, rx, ry, angle=0., **kw ):
		self.kw_init( kw )
		self.p = p
		self.rx = rx
		self.ry = ry
		self.angle = angle

	def bbox( self, context ):
		r = self.rx, self.ry
		p = pt_add( self.p, r )
		q = pt_sub( self.p, r )
		bb = BoundingBox( p, q )
		bb.rotate( self.p, self.angle )
		return bb

	def draw( self, context ):
		context.draw.ellipse( self.p, self.rx, self.ry, self.angle )

class _CombObject( _DeviceObject ):

	def __init__( self, points, dp, **kw ):
		self.kw_init( kw )
		self.points = points
		self.dp = dp

	def bbox( self, context ):
		return apply( BoundingBox, self.points )

	def draw( self, context ):
		for p in self.points:
			context.draw.move( p )
			context.draw.linetorel( self.dp )

class _BoxObject( _DeviceObject ):

	def __init__( self, p, q, **kw ):
		self.kw_init( kw )
		self.p = p
		self.q = q

	def bbox( self, context ):
		return BoundingBox( self.p, self.q )

	def draw( self, context ):
		context.draw.rect( self.p, self.q )

class _ArcObject( _DeviceObject ):

	def __init__( self, pc, p0, p1, **kw ):
		self.kw_init( kw )
		self.pc = pc
		self.p0 = p0
		self.p1 = p1

	def bbox( self, context ):
		return BoundingBox( self.pc, self.p0, self.p1 )

	def draw( self, context ):
		context.draw.arc( self.pc, self.p0, self.p1 )

# _PlotComponent --------------------------------------------------------------

class _PlotComponent( _StyleKeywords, _ConfAttributes ):

	def __init__( self ):
		self.clear()

	def add( self, *args ):
		for obj in args:
			self.device_objects.append( obj )

	def limits( self ):
		return BoundingBox()

	def clear( self ):
		self.device_objects = []

	def make( self, context ):
		raise BigglesError

	def make_key( self, bbox ):
		pass

	def bbox( self, context ):
		self.clear()
		self.make( context )
		bb = BoundingBox()
		for obj in self.device_objects:
			bb.union( obj.bbox(context) )
		return bb

	def render( self, context ):
		self.clear()
		self.make( context )
		self.kw_predraw( context )
		for obj in self.device_objects:
			obj.render( context )
		self.kw_postdraw( context )

# _LabelComponent -------------------------------------------------------------

class _LabelComponent( _PlotComponent ):

	kw_rename = {
		'face'		: 'fontface',
		'size'		: 'fontsize',
		'angle'		: 'textangle',
		'halign'	: 'texthalign',
		'valign'	: 'textvalign',
	}

	def __init__( self, x, y, str, **kw ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "_LabelComponent" )
		self.kw_init( kw )
		self.pos = x, y
		self.str = str

	def limits( self ):
		return BoundingBox()

class DataLabel( _LabelComponent ):

	def make( self, context ):
		pos = apply( context.geom, self.pos )
		t = apply( _TextObject, (pos, self.str), self.kw_style )
		self.add( t )

class PlotLabel( _LabelComponent ):

	def make( self, context ):
		pos = apply( context.plot_geom, self.pos )
		t = apply( _TextObject, (pos, self.str), self.kw_style )
		self.add( t )

# _LineComponent --------------------------------------------------------------

class _LineComponent( _PlotComponent ):

	def __init__( self ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "_LineComponent" )

	kw_rename = {
		'color' : 'linecolor',
		'width' : 'linewidth',
		'type' : 'linetype',
	}

	def make_key( self, bbox ):
		xr = bbox.xrange()
		y = bbox.center()[1]
		p = xr[0], y
		q = xr[1], y
		return apply( _LineObject, (p,q), self.kw_style )

class Curve( _LineComponent ):

	def __init__( self, x, y, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "Curve" )
		self.kw_init( kw )
		self.x = x
		self.y = y

	def limits( self ):
		p0 = min(self.x), min(self.y)
		p1 = max(self.x), max(self.y)
		return BoundingBox( p0, p1 )

	def make( self, context ):
		segs = context.geom.geodesic( self.x, self.y )
		for seg in segs:
			x, y = context.geom.call_vec( seg[0], seg[1] )
			self.add( _PathObject(x, y) )

class DataLine( _LineComponent ):

	def __init__( self, p, q, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "DataLine" )
		self.kw_init( kw )
		self.p = p
		self.q = q

	def limits( self ):
		return BoundingBox( self.p, self.q )

	def make( self, context ):
		a = apply( context.geom, self.p )
		b = apply( context.geom, self.q )
		self.add( _LineObject(a, b) )

class Geodesic( _LineComponent ):

	def __init__( self, p, q, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "Geodesic" )
		self.kw_init( kw )
		self.p = p
		self.q = q

	def limits( self ):
		return BoundingBox( self.p, self.q )

	def make( self, context ):
		l = self.p[0], self.q[0]
		b = self.p[1], self.q[1]
		segs = context.geom.geodesic( l, b, self.divisions )
		for seg in segs:
			x, y = context.geom.call_vec( seg[0], seg[1] )
			self.add( _PathObject(x, y) )

class Histogram( _LineComponent ):

	def __init__( self, values, x0=0, binsize=1, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "Histogram" )
		self.kw_init( kw )
		self.values = values
		self.x0 = x0
		self.binsize = binsize

	def limits( self ):
		nval = len( self.values )
		if self.drop_to_zero:
			p = self.x0, min( 0, min(self.values) )
		else:
			p = self.x0, min(self.values)
		q = self.x0 + nval*self.binsize, max(self.values)
		return BoundingBox( p, q )

	def make( self, context ):
		nval = len( self.values )
		x = []
		y = []
		if self.drop_to_zero:
			x.append( self.x0 )
			y.append( 0 )
		for i in range(0,nval):
			xi = self.x0 + i * self.binsize
			yi = self.values[i]
			x.extend( [xi, xi + self.binsize] )
			y.extend( [yi, yi] )
		if self.drop_to_zero:
			x.append( self.x0 + nval*self.binsize )
			y.append( 0 )
		u, v = context.geom.call_vec( x, y )
		self.add( _PathObject(u, v) )

class LineX( _LineComponent ):

	def __init__( self, x, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "LineX" )
		self.kw_init( kw )
		self.x = x

	def limits( self ):
		return BoundingBox( (self.x,None), (self.x,None) )

	def make( self, context ):
		yrange = context.data_bbox.yrange()
		p = self.x, yrange[0]
		q = self.x, yrange[1]
		a = apply( context.geom, p )
		b = apply( context.geom, q )
		self.add( _LineObject(a, b) )

class LineY( _LineComponent ):

	def __init__( self, y, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "LineY" )
		self.kw_init( kw )
		self.y = y

	def limits( self ):
		return BoundingBox( (None,self.y), (None,self.y) )

	def make( self, context ):
		xrange = context.data_bbox.xrange()
		p = xrange[0], self.y
		q = xrange[1], self.y
		a = apply( context.geom, p )
		b = apply( context.geom, q )
		self.add( _LineObject(a, b) )

class PlotLine( _LineComponent ):

	def __init__( self, p, q, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "PlotLine" )
		self.kw_init( kw )
		self.p = p
		self.q = q

	def make( self, context ):
		a = apply( context.plot_geom, self.p )
		b = apply( context.plot_geom, self.q )
		self.add( _LineObject(a, b) )

class Slope( _LineComponent ):

	def __init__( self, slope, intercept=None, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "Slope" )
		self.kw_init( kw )
		self.slope = slope
		self.intercept = intercept
		if intercept is None:
			self.intercept = (0.,0.)

	def _x( self, y ):
		x0, y0 = self.intercept
		return x0 + float(y - y0) / self.slope

	def _y( self, x ):
		x0, y0 = self.intercept
		return y0 + (x - x0) * self.slope

	def make( self, context ):
		xrange = context.data_bbox.xrange()
		yrange = context.data_bbox.yrange()
		if self.slope == 0:
			l = [ ( xrange[0], self.intercept[1] ),\
			      ( xrange[1], self.intercept[1] ) ]
		else:
			l = [ ( xrange[0], self._y(xrange[0]) ),\
			      ( xrange[1], self._y(xrange[1]) ),\
			      ( self._x(yrange[0]), yrange[0] ),\
			      ( self._x(yrange[1]), yrange[1] ) ]
		m = filter( context.data_bbox.contains, l )
		m.sort()
		if len(m) > 1:
			a = apply( context.geom, m[0] )
			b = apply( context.geom, m[-1] )
			self.add( _LineObject(a, b) )

class DataBox( _LineComponent ):

	def __init__( self, p, q, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "DataBox" )
		self.kw_init( kw )
		self.p = p
		self.q = q

	def limits( self ):
		return BoundingBox( self.p, self.q )

	def make( self, context ):
		a = apply( context.geom, self.p )
		b = apply( context.geom, self.q )
		self.add( _BoxObject(a, b) )

class PlotBox( _LineComponent ):

	def __init__( self, p, q, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "PlotBox" )
		self.kw_init( kw )
		self.p = p
		self.q = q

	def make( self, context ):
		a = apply( context.plot_geom, self.p )
		b = apply( context.plot_geom, self.q )
		self.add( _BoxObject(a, b) )

class PlotArc( _LineComponent ):

	def __init__( self, pc, r, a0, a1, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "PlotArc" )
		self.kw_init( kw )
		self.pc = pc
		self.p0 = p[0] + r*math.cos(a0), p[1] + r*math.sin(a0)
		self.p1 = p[0] + r*math.cos(a1), p[1] + r*math.sin(a1)

	def make( self, context ):
		pc = apply( context.plot_geom, self.pc )
		p0 = apply( context.plot_geom, self.p0 )
		p1 = apply( context.plot_geom, self.p1 )
		self.add( _ArcObject(pc, p0, p1) )

class DataArc( _LineComponent ):

	def __init__( self, pc, r, a0, a1, **kw ):
		_LineComponent.__init__( self )
		self.conf_setattr( "DataArc" )
		self.kw_init( kw )
		self.pc = pc
		self.p0 = p[0] + r*math.cos(a0), p[1] + r*math.sin(a0)
		self.p1 = p[0] + r*math.cos(a1), p[1] + r*math.sin(a1)

	def limits( self ):
		return BoundingBox( self.pc, self.p0, self.p1 )

	def make( self, context ):
		pc = apply( context.geom, self.pc )
		p0 = apply( context.geom, self.p0 )
		p1 = apply( context.geom, self.p1 )
		self.add( _ArcObject(pc, p0, p1) )

# _SymbolDataComponent --------------------------------------------------------

class _SymbolDataComponent( _PlotComponent ):

	kw_rename = {
		'type' : 'symboltype',
		'size' : 'symbolsize',
	}

	def make_key( self, bbox ):
		pos = bbox.center()
		return apply(_SymbolObject, (pos,), self.kw_style)

class Points( _SymbolDataComponent ):

	kw_defaults = {
		'symboltype' : config.value('Points','symboltype'),
		'symbolsize' : config.value('Points','symbolsize'),
	}

	def __init__( self, x, y, **kw ):
		_SymbolDataComponent.__init__( self )
		self.conf_setattr( "Points" )
		self.kw_init( kw )
		self.x = x
		self.y = y

	def limits( self ):
		p = min(self.x), min(self.y)
		q = max(self.x), max(self.y)
		return BoundingBox( p, q )

	def make( self, context ):
		x, y = context.geom.call_vec( self.x, self.y )
		self.add( _SymbolsObject(x, y) )

def Point( x, y, **kw ):
	return apply( Points, ([x],[y]), kw )

class ColoredPoints( _SymbolDataComponent ):

	kw_defaults = {
		'symboltype' : config.value('Points','symboltype'),
		'symbolsize' : config.value('Points','symbolsize'),
	}

	def __init__( self, x, y, c=None, **kw ):
		_SymbolDataComponent.__init__( self )
		self.conf_setattr( "Points" )
		self.kw_init( kw )
		self.x = x
		self.y = y
		self.c = c

	def limits( self ):
		p = min(self.x), min(self.y)
		q = max(self.x), max(self.y)
		return BoundingBox( p, q )

	def make( self, context ):
		x, y = context.geom.call_vec( self.x, self.y )
		self.add( _ColoredSymbolsObject(x, y, self.c) )

def ColoredPoint( x, y, **kw ):
	return apply( ColoredPoints, ([x],[y]), kw )

# _DensityComponent -----------------------------------------------------------

class Density( _PlotComponent ):

	kw_defaults = {
		'foo' : config.value('Points','symbolsize'),
	}

	def __init__( self, densgrid, ((xmin,ymin), (xmax,ymax)), **kw ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "Density" )
		self.kw_init( kw )
		self.densgrid = densgrid
		self.extent   = ((xmin,ymin), (xmax,ymax))

	def limits( self ):
		return apply( BoundingBox, self.extent )

	def make( self, context ):
		(x0,y0),(x1,y1) = self.extent
		(x0,x1),(y0,y1) = context.geom.call_vec((x0,x1),(y0,y1))
		self.add( _DensityObject(self.densgrid, ((x0,y0),(x1,y1))) )

# _FillComponent --------------------------------------------------------------

class _FillComponent( _PlotComponent ):

	kw_defaults = {
		'color' : config.value('_FillComponent','fillcolor'),
		'filltype' : config.value('_FillComponent','filltype'),
	}

	def make_key( self, bbox ):
		p = bbox.lowerleft()
		q = bbox.upperright()
		return apply( _BoxObject, (p,q), self.kw_style )

class FillAbove( _FillComponent ):

	def __init__( self, x, y, **kw ):
		_FillComponent.__init__( self )
		self.conf_setattr( "FillAbove" )
		self.kw_init( kw )
		self.x = x
		self.y = y

	def limits( self ):
		p = min(self.x), min(self.y)
		q = max(self.x), max(self.y)
		return BoundingBox( p, q )

	def make( self, context ):
		coords = map( context.geom, self.x, self.y )
		max_y = context.data_bbox.yrange()[1]
		coords.append( context.geom(self.x[-1], max_y) )
		coords.append( context.geom(self.x[0], max_y) )
		self.add( _PolygonObject(coords) )

class FillBelow( _FillComponent ):

	def __init__( self, x, y, **kw ):
		_FillComponent.__init__( self )
		self.conf_setattr( "FillBelow" )
		self.kw_init( kw )
		self.x = x
		self.y = y

	def limits( self ):
		p = min(self.x), min(self.y)
		q = max(self.x), max(self.y)
		return BoundingBox( p, q )

	def make( self, context ):
		coords = map( context.geom, self.x, self.y )
		min_y = context.data_bbox.yrange()[0]
		coords.append( context.geom(self.x[-1], min_y) )
		coords.append( context.geom(self.x[0], min_y) )
		self.add( _PolygonObject(coords) )

class FillBetween( _FillComponent ):

	def __init__( self, x1, y1, x2, y2, **kw ):
		_FillComponent.__init__( self )
		self.conf_setattr( "FillBetween" )
		self.kw_init( kw )
		self.x1, self.y1 = x1, y1
		self.x2, self.y2 = x2, y2

	def limits( self ):
		min_x = min( min(self.x1), min(self.x2) )
		max_x = max( max(self.x1), max(self.x2) )
		min_y = min( min(self.y1), min(self.y2) )
		max_y = max( max(self.y1), max(self.y2) )
		return BoundingBox( (min_x,min_y), (max_x,max_y) )

	def make( self, context ):
		x = list(self.x1) + _tsil(self.x2)
		y = list(self.y1) + _tsil(self.y2)
		coords = map( context.geom, x, y )
		self.add( _PolygonObject(coords) )

# ErrorBars -------------------------------------------------------------------

class _ErrorBar( _PlotComponent ):

	kw_rename = {
		'color' : 'linecolor',
		'width' : 'linewidth',
		'type' : 'linetype',
	}

	def __init__( self ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "_ErrorBar" )

class ErrorBarsX( _ErrorBar ):

	def __init__( self, y, lo, hi, **kw ):
		_ErrorBar.__init__( self )
		self.conf_setattr( "ErrorBarsX" )
		self.kw_init( kw )
		self.y = y
		self.lo = lo
		self.hi = hi

	def limits( self ):
		p = min( min(self.lo), min(self.hi) ), min(self.y)
		q = max( max(self.lo), max(self.hi) ), max(self.y)
		return BoundingBox( p, q )

	def make( self, context ):
		l = _size_relative( self.barsize, context.dev_bbox ) 
		for i in range(len(self.y)):
			p = context.geom( self.lo[i], self.y[i] )
			q = context.geom( self.hi[i], self.y[i] )
			l0 = _LineObject( p, q )
			l1 = _LineObject( (p[0],p[1]-l), (p[0],p[1]+l) )
			l2 = _LineObject( (q[0],q[1]-l), (q[0],q[1]+l) )
			self.add( l0, l1, l2 )

class ErrorBarsY( _ErrorBar ):

	def __init__( self, x, lo, hi, **kw ):
		_ErrorBar.__init__( self )
		self.conf_setattr( "ErrorBarsY" )
		self.kw_init( kw )
		self.x = x
		self.lo = lo
		self.hi = hi

	def limits( self ):
		p = min(self.x), min( min(self.lo), min(self.hi) )
		q = max(self.x), max( max(self.lo), max(self.hi) )
		return BoundingBox( p, q )

	def make( self, context ):
		l = _size_relative( self.barsize, context.dev_bbox )
		for i in range(len(self.x)):
			p = context.geom( self.x[i], self.lo[i] )
			q = context.geom( self.x[i], self.hi[i] )
			l0 = _LineObject( p, q )
			l1 = _LineObject( (p[0]-l,p[1]), (p[0]+l,p[1]) )
			l2 = _LineObject( (q[0]-l,q[1]), (q[0]+l,q[1]) )
			self.add( l0, l1, l2 )

def SymmetricErrorBarsX( x, y, err, **kw ):
	import operator
	xlo = map( operator.sub, x, err )
	xhi = map( operator.add, x, err )
	return apply( ErrorBarsX, (y, xlo, xhi), kw )

def SymmetricErrorBarsY( x, y, err, **kw ):
	import operator
	ylo = map( operator.sub, y, err )
	yhi = map( operator.add, y, err )
	return apply( ErrorBarsY, (x, ylo, yhi), kw )

# Limits ----------------------------------------------------------------------

class _ErrorLimit( _PlotComponent ):

	kw_rename = {
		'color' : 'linecolor',
		'width' : 'linewidth',
		'type' : 'linetype',
	}

	def __init__( self ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "_ErrorLimit" )

class UpperLimits( _ErrorLimit ):

	def __init__( self, x, ulimit, **kw ):
		_ErrorLimit.__init__( self )
		self.conf_setattr( "UpperLimits" )
		self.kw_init( kw )
		self.x = x
		self.ulimit = ulimit

	def limits( self ):
		p = min(self.x), min(self.ulimit)
		q = max(self.x), max(self.ulimit)
		return BoundingBox( p, q )

	def make( self, context ):
		l = _size_relative( self.size, context.dev_bbox )
		for i in range(len(self.x)):
			p = context.geom( self.x[i], self.ulimit[i] )
			l1 = _LineObject( (p[0]-l,p[1]), (p[0]+l,p[1]) )
			l2 = _LineObject( (p[0],p[1]-2*l), (p[0],p[1]) )
			l3 = _LineObject( (p[0],p[1]-2*l), (p[0]+l,p[1]-l) )
			l4 = _LineObject( (p[0],p[1]-2*l), (p[0]-l,p[1]-l) )
			self.add( l1, l2, l3, l4 )

class LowerLimits( _ErrorLimit ):

	def __init__( self, x, llimit, **kw ):
		_ErrorLimit.__init__( self )
		self.conf_setattr( "UpperLimits" )
		self.kw_init( kw )
		self.x = x
		self.llimit = llimit

	def limits( self ):
		p = min(self.x), min(self.llimit)
		q = max(self.x), max(self.llimit)
		return BoundingBox( p, q )

	def make( self, context ):
		l = _size_relative( self.size, context.dev_bbox )
		for i in range(len(self.x)):
			p = context.geom( self.x[i], self.llimit[i] )
			l1 = _LineObject( (p[0]-l,p[1]), (p[0]+l,p[1]) )
			l2 = _LineObject( (p[0],p[1]+2*l), (p[0],p[1]) )
			l3 = _LineObject( (p[0],p[1]+2*l), (p[0]+l,p[1]+l) )
			l4 = _LineObject( (p[0],p[1]+2*l), (p[0]-l,p[1]+l) )
			self.add( l1, l2, l3, l4 )

# Ellipses --------------------------------------------------------------------

class Ellipses( _PlotComponent ):

	kw_rename = {
		'color' : 'linecolor',
		'width' : 'linewidth',
		'type' : 'linetype',
	}

	def __init__( self, x, y, rx, ry, angle=None, **kw ):
		_PlotComponent.__init__( self )
		self.kw_init( kw )
		self.x = x
		self.y = y
		self.rx = rx
		self.ry = ry
		self.angle = angle

	def limits( self ):
		# XXX:kludge
		minx, maxx = self.x[0], self.x[0]
		miny, maxy = self.y[0], self.y[0]
		for i in range(len(self.x)):
			r = max( self.rx[i], self.ry[i] )
			minx = min( minx, self.x[i]-r )
			miny = min( miny, self.y[i]-r )
			maxx = max( maxx, self.x[i]+r )
			maxy = max( maxy, self.y[i]+r )
		return BoundingBox( (minx,miny), (maxx,maxy) )

	def make( self, context ):
		for i in range(len(self.x)):
			p = context.geom( self.x[i], self.y[i] )
			r = context.geom( \
				self.x[i] + self.rx[i], \
				self.y[i] + self.ry[i] )
			rx, ry = pt_sub( r, p )
			
			if self.angle is not None:
				e = _EllipseObject( p, rx, ry, self.angle[i] )
			else:
				e = _EllipseObject( p, rx, ry )
			self.add( e )

def Ellipse( x, y, rx, ry, angle=None, **kw ):
	if angle is None:
		args = ([x],[y],[rx],[ry])
	else:
		args = ([x],[y],[rx],[ry],[angle])
	return apply( Ellipses, args, kw )

def Circles( x, y, r, **kw ):
	return apply( Ellipses, (x,y,r,r), kw )

def Circle( x, y, r, **kw ):
	return apply( Circles, ([x],[y],[r]), kw )

# _PlotKey --------------------------------------------------------------------

class PlotKey( _PlotComponent ):

	kw_rename = {
		'face'		: 'fontface',
		'size'		: 'fontsize',
		'angle'		: 'textangle',
		'halign'	: 'texthalign',
		'valign'	: 'textvalign',
	}

	def __init__( self, x, y, components, **kw ):
		_PlotComponent.__init__( self )
		self.conf_setattr( "PlotKey" )
		self.kw_init( kw )
		self.x = x
		self.y = y
		self.components = components

	def make( self, context ):
		key_pos = context.plot_geom( self.x, self.y )
		key_width = _size_relative( self.key_width, context.dev_bbox )
		key_height = _size_relative( self.key_height, context.dev_bbox )
		key_hsep = _size_relative( self.key_hsep, context.dev_bbox )
		key_vsep = _size_relative( self.key_vsep, context.dev_bbox )

		halign = self.kw_get( 'texthalign' )
		if halign == 'left':
			text_pos = pt_add( (key_width/2+key_hsep,0), key_pos )
		else:
			text_pos = pt_add( (-key_width/2-key_hsep,0), key_pos )
		bbox = BoundingBox( (-key_width/2,-key_height/2),
			(key_width/2,key_height/2) )
		bbox.shift( key_pos )
		dp = 0, -(key_vsep + key_height)

		for comp in self.components:
			try:
				obj,str = comp
			except:
				obj = comp
				str = getattr( comp, "label", "" )
			t = apply( _TextObject, (text_pos,str), self.kw_style )
			self.add( t, obj.make_key(bbox) )
			text_pos = pt_add( text_pos, dp )
			bbox.shift( dp )

# XXX:deprecated
def OldKey( x, y, labels, align='left', **kw ):
	kw['texthalign'] = align
	return apply( PlotKey, (x,y,labels), kw )

# _HalfAxis -------------------------------------------------------------------

def _magform( x ):
	"Given x, returns (a,b), where x = a*10^b [a >= 1., b integral]."
	if x == 0:
		return 0., 0
	a, b = math.modf(math.log10(abs(x)))
	a, b = math.pow(10,a), int(b)
	if a < 1.:
		a, b = a * 10, b - 1
	if x < 0.:
		a = -a
	return a, b

def _format_ticklabel( x, range=0. ):
	if x == 0:
		return "0"
	a, b = _magform( x )
	if abs(b) > 4:
		if a == 1.:
			return r"$10^{%d}$" % b
		elif a == -1.:
			return r"-$10^{%d}$" % b
		else:
			return r"$%g\times 10^{%d}$" % (a,b)
	if range < 1e-6:
		a, b = _magform( range )
		return "%.*f" % (abs(b),x)
	return "%g" % x

def _ticklist_linear( lo, hi, sep, origin=0. ):
	r = []
	a = _ceil(float(lo - origin)/float(sep))
	b = _floor(float(hi - origin)/float(sep))
	#for i in range( a, b+1 ):
	#	r.append( origin + i * sep )
	r0 = origin + a*sep
	for i in range( b-a+1 ):
		r.append( r0 + i*sep )
	return r

def _pow10(x):
	return math.pow(10,x)

def _log10(x):
	return math.log10(x)

def _ticks_default_linear( lim ):
	a, b = _magform( (lim[1] - lim[0])/5. )
	if a < (1 + 2)/2.:
		x = 1
	elif a < (2 + 5)/2.:
		x = 2
	elif a < (5 + 10)/2.:
		x = 5
	else:
		x = 10

	major_div = x * math.pow(10, b)
	return _ticklist_linear( lim[0], lim[1], major_div )

def _ticks_default_log( lim ):
	log_lim = _log10(lim[0]), _log10(lim[1])
	nlo = _ceil( math.log10(lim[0]) )
	nhi = _floor( math.log10(lim[1]) )
	nn = nhi - nlo +1

	if nn >= 10:
		return map( _pow10, _ticks_default_linear(log_lim) )
	elif nn >= 2:
		return map( _pow10, range(nlo, nhi+1) )
	else:
		return _ticks_default_linear( lim )

def _ticks_num_linear( lim, num ):
	ticks = []
	a = lim[0]
	b = (lim[1] - lim[0])/float(num-1)
	for i in range(num):
		ticks.append( a + i*b )
	return ticks

def _ticks_num_log( lim, num ):
	ticks = []
	a = math.log10(lim[0])
	b = (math.log10(lim[1]) - a)/float(num - 1)
	for i in range(num):
		ticks.append( a + i*b )
	return map( _pow10, ticks )

def _subticks_linear( lim, ticks, num=None ):
	major_div = (ticks[-1] - ticks[0])/float(len(ticks) - 1)
	if num is None:
		_num = 4
		a, b = _magform( major_div )
		if 1. < a < (2 + 5)/2.:
			_num = 3
	else:
		_num = num
	minor_div = major_div/float(_num+1)
	return _ticklist_linear( lim[0], lim[1], minor_div, ticks[0] )

def _subticks_log( lim, ticks, num=None ):
	log_lim = _log10(lim[0]), _log10(lim[1])
	nlo = _ceil( math.log10(lim[0]) )
	nhi = _floor( math.log10(lim[1]) )
	nn = nhi - nlo +1

	if nn >= 10:
		return map( _pow10, _subticks_linear(log_lim, map(_log10,ticks), num) )
	elif nn >= 2:
		minor_ticks = []
		for i in range(nlo-1,nhi+1):
			for j in range(1,10):
				z = j * _pow10(i)
				if lim[0] <= z and z <= lim[1]:
					minor_ticks.append(z)
		return minor_ticks
	else:
		return _subticks_linear( lim, ticks, num )

class _Group:

	def __init__( self, objs ):
		self.objs = objs[:]
	
	def bbox( self, context ):
		bb = BoundingBox()
		for obj in self.objs:
			bb.union( obj.bbox(context) )
		return bb

class _HalfAxis( _PlotComponent ):

	func_ticks_default	= _ticks_default_linear, _ticks_default_log
	func_ticks_num		= _ticks_num_linear, _ticks_num_log

	func_subticks_default	= _subticks_linear, _subticks_log
	func_subticks_num	= _subticks_linear, _subticks_log

	_attr_map = {
		'labeloffset'		: 'label_offset',
		'major_ticklabels'	: 'ticklabels',
		'major_ticks'		: 'ticks',
		'minor_ticks'		: 'subticks',
	}

	def __init__( self, **kw ):
		_PlotComponent.__init__( self )
		self.kw_init( kw )
		self.conf_setattr( "_HalfAxis" )

	def __getattr__( self, name ):
		return self.__dict__[ self._attr_map.get(name,name) ]

	def __setattr__( self, name, value ):
		self.__dict__[ self._attr_map.get(name,name) ] = value

	def _ticks( self, context ):
		log = self._log( context )
		_range = self._range( context )
		if self.ticks is None:
			return self.func_ticks_default[log]( _range )
		elif type(self.ticks) == type(0):
			return self.func_ticks_num[log]( _range, self.ticks )
		else:
			return self.ticks

	def _subticks( self, context, ticks ):
		log = self._log( context )
		_range = self._range( context )
		if self.subticks is None:
			return self.func_subticks_default[log]( _range, ticks )
		elif type(self.subticks) == type(0):
			return self.func_subticks_num[log]( \
				_range, ticks, self.subticks )
		else:
			return self.subticks

	def _ticklabels( self, context, ticks ):
		if self.ticklabels is None:
			range = [ max(ticks) - min(ticks) ] * len(ticks)
			return map( _format_ticklabel, ticks, range )
		else:
			return self.ticklabels

	def _make_ticklabels( self, context, pos, labels ):
		if labels is None or not len(labels) > 0:
			return

		dir = self.ticklabels_dir
		offset = _size_relative( self.ticklabels_offset, \
			context.dev_bbox )
		if self.draw_ticks and self.tickdir > 0:
			offset = offset + _size_relative( \
				self.ticks_size, context.dev_bbox )
		labelpos = []
		for i in range(len(labels)):
			labelpos.append( \
				self._pos(context, pos[i], dir*offset) )

		halign, valign = self._align()

		style = { "halign" : halign, "valign" : valign }
		style.update( self.ticklabels_style )

		l = apply( _LabelsObject, (labelpos, labels), style )
		self.add( l )

	def _make_spine( self, context ):
		a, b = self._range( context )
		p = self._pos( context, a )
		q = self._pos( context, b )
		self.add( apply(_LineObject, (p, q), self.spine_style) )

	def _make_ticks( self, context, ticks, size, style ):
		if ticks is None or not len(ticks) > 0:
			return

		dir = self.tickdir * self.ticklabels_dir
		ticklen = self._dpos( dir *\
			_size_relative(size, context.dev_bbox) )
		tickpos = []
		for tick in ticks:
			tickpos.append( self._pos(context, tick) )

		self.add( apply(_CombObject, (tickpos, ticklen), style) )

	def make( self, context ):
		if self.draw_nothing:
			return

		ticks = self._ticks( context )
		subticks = self._subticks( context, ticks )
		ticklabels = self._ticklabels( context, ticks )

		implicit_draw_subticks = self.draw_subticks is None and \
			self.draw_ticks

		implicit_draw_ticklabels = self.draw_ticklabels is None and \
			(self.range is not None or self.ticklabels is not None)

		if self.draw_grid:
			self._make_grid( context, ticks )

		if self.draw_axis:
			if self.draw_subticks or implicit_draw_subticks:
				self._make_ticks( context, subticks, \
					self.subticks_size, \
					self.subticks_style )

			if self.draw_ticks:
				self._make_ticks( context, ticks, \
					self.ticks_size, self.ticks_style )

			if self.draw_spine:
				self._make_spine( context )

		if self.draw_ticklabels or implicit_draw_ticklabels:
			self._make_ticklabels( context, ticks, ticklabels )

		## has to be made last
		if self.label is not None:
			self.add( apply(_BoxLabel,
				(_Group(self.device_objects),
				self.label, self._side(), self.label_offset),\
				self.label_style) )

class _HalfAxisX( _HalfAxis ):

	def _pos( self, context, a, db=0. ):
		p = context.geom( a, self._intercept(context) )
		return p[0], p[1] + db

	def _dpos( self, d ):
		return 0., d

	def _align( self ):
		if self.ticklabels_dir < 0:
			return 'center', 'top'
		else:
			return 'center', 'bottom'

	def _intercept( self, context ):
		if self.intercept is not None:
			return self.intercept
		limits = context.data_bbox
		if self.ticklabels_dir < 0:
			return limits.yrange()[0]
		else:
			return limits.yrange()[1]

	def _log( self, context ):
		if self.log is None:
			return context.xlog
		return self.log

	def _side( self ):
		if self.ticklabels_dir < 0:
			return 'bottom'
		else:
			return 'top'

	def _range( self, context ):
		if self.range is not None:
			a,b = self.range
			if a is None or b is None:
				c,d = context.data_bbox.xrange()
				if a is None: a = c
				if b is None: b = d
				return a,b
			else:
				return self.range
		return context.data_bbox.xrange()

	def _make_grid( self, context, ticks ):
		if ticks is None:
			return
		for tick in ticks:
			self.add( apply(LineX, (tick,), self.grid_style) )

class _HalfAxisY( _HalfAxis ):

	def _pos( self, context, a, db=0. ):
		p = context.geom( self._intercept(context), a )
		return p[0] + db, p[1]

	def _dpos( self, d ):
		return d, 0.

	def _align( self ):
		if self.ticklabels_dir > 0:
			return 'left', 'center'
		else:
			return 'right', 'center'

	def _intercept( self, context ):
		if self.intercept is not None:
			return self.intercept
		limits = context.data_bbox
		if self.ticklabels_dir > 0:
			return limits.xrange()[1]
		else:
			return limits.xrange()[0]

	def _log( self, context ):
		if self.log is None:
			return context.ylog
		return self.log

	def _side( self ):
		if self.ticklabels_dir > 0:
			return 'right'
		else:
			return 'left'

	def _range( self, context ):
		if self.range is not None:
			a,b = self.range
			if a is None or b is None:
				c,d = context.data_bbox.yrange()
				if a is None: a = c
				if b is None: b = d
				return a,b
			else:
				return self.range
		return context.data_bbox.yrange()

	def _make_grid( self, context, ticks ):
		if ticks is None:
			return
		for tick in ticks:
			self.add( apply(LineY, (tick,), self.grid_style) )

# _BoxLabel -------------------------------------------------------------------

class _BoxLabel( _PlotComponent ):

	kw_rename = {
		'face' : 'fontface',
		'size' : 'fontsize',
	}

	def __init__( self, obj, str, side, offset, **kw ):
		_PlotComponent.__init__( self )
		self.kw_init( kw )
		self.obj = obj
		self.str = str
		self.side = side
		self.offset = offset

	def make( self, context ):
		bb = self.obj.bbox( context )
		offset = _size_relative( self.offset, context.dev_bbox )
		if self.side == 'top':
			p = bb.upperleft()
			q = bb.upperright()
		elif self.side == 'bottom':
			p = bb.lowerleft()
			q = bb.lowerright()
			offset = -offset
		elif self.side == 'left':
			p = bb.lowerleft()
			q = bb.upperleft()
		elif self.side == 'right':
			p = bb.upperright()
			q = bb.lowerright()

		lt = apply( _LineTextObject, (p, q, self.str, offset), \
			self.kw_style )
		self.add( lt )

# _PlotComposite --------------------------------------------------------------

class _PlotComposite( _StyleKeywords ):

	def __init__( self, **kw ):
		self.kw_init( kw )
		self.components = []
		self.dont_clip = 0

	def add( self, *args ):
		for obj in args:
			self.components.append( obj )

	def clear( self ):
		self.components = []

	def empty( self ):
		return len(self.components) == 0

	def limits( self ):
		bb = BoundingBox()
		for obj in self.components:
			bb.union( obj.limits() )
		return bb

	def make( self, context ):
		pass

	def bbox( self, context ):
		self.make( context )
		bb = BoundingBox()
		for obj in self.components:
			bb.union( obj.bbox(context) )
		return bb

	def render( self, context ):
		self.make( context )
		self.kw_predraw( context )
		if not self.dont_clip:
			context.do_clip()
		for obj in self.components:
			obj.render( context )
		self.kw_postdraw( context )

# Frame -----------------------------------------------------------------------

class Frame( _PlotComposite ):

	def __init__( self, labelticks=(0,1,1,0), **kw ):
		apply( _PlotComposite.__init__, (self,), kw )
		self.dont_clip = 1

		self.x2 = _HalfAxisX()
		self.x2.draw_ticklabels = labelticks[0]
		self.x2.ticklabels_dir = 1

		self.x1 = _HalfAxisX()
		self.x1.draw_ticklabels = labelticks[1]
		self.x1.ticklabels_dir = -1
		
		self.y1 = _HalfAxisY()
		self.y1.draw_ticklabels = labelticks[2]
		self.y1.ticklabels_dir = -1

		self.y2 = _HalfAxisY()
		self.y2.draw_ticklabels = labelticks[3]
		self.y2.ticklabels_dir = 1

	def make( self, context ):
		self.clear()
		self.add( self.x1, self.x2, self.y1, self.y2 )

# _PlotContainer --------------------------------------------------------------

def _open_output( filename ):
	if filename == '-':
		import sys
		return sys.stdout
	else:
		# b is for windows
		return open( filename, 'wb' )

def _close_output( file ):
	if 2 < file.fileno():
		file.close()
	else:
		file.flush()

def _draw_text( device, p, str, **kw ):
	device.save_state()
	for key,val in kw.items():
		device.set( key, val )
	device.text( p, str )
	device.restore_state()

def win_temp_path():
	"""
	Intended for Windows, returns a valid temp directory, or
	at least the current working directory.
	"""
	import os
	if os.environ.has_key('TEMP'):
		if os.path.exists(os.environ['TEMP']):
			return(os.environ['TEMP'])
	else:
		possible_temp_paths = [ '\\temp', '\\winnt\\temp', \
			'\\windows\\temp', '\\tmp' ]
		for i in possible_temp_paths:
			if os.path.exists(i):
				return i
	return os.getcwd()

class _PlotContainer( _ConfAttributes ):

	def __init__( self, **kw ):
		apply( self.conf_setattr, ("_PlotContainer",), kw )

	def empty( self ):
		pass

	def interior( self, device, exterior ):
		TOL = 0.005

		interior = exterior.copy()
		region_diagonal = exterior.diagonal()

		for i in range(10):
			bb = self.exterior( device, interior )

			dll = pt_sub( exterior.lowerleft(), bb.lowerleft() )
			dur = pt_sub( exterior.upperright(), bb.upperright() )

			sll = pt_len(dll) / region_diagonal
			sur = pt_len(dur) / region_diagonal

			if sll < TOL and sur < TOL:
				# XXX:fixme
				if self.aspect_ratio is not None:
					interior.make_aspect_ratio(\
						self.aspect_ratio )
				return interior

			scale = interior.diagonal() / bb.diagonal()
			dll = pt_mul( scale, dll )
			dur = pt_mul( scale, dur )

			interior = BoundingBox(
				pt_add(interior.lowerleft(), dll),
				pt_add(interior.upperright(), dur) )

		raise BigglesError

	def exterior( self, device, interior ):
		return interior.copy()

	def compose_interior( self, device, interior ):
		if self.title is not None:
			offset = _size_relative( self.title_offset, interior )
			exterior = self.exterior( device, interior )
			x = interior.center()[0]
			y = exterior.yrange()[1] + offset
			style = self.title_style.copy()
			style["fontsize"] = _fontsize_relative( \
				self.title_style["fontsize"], interior, device )
			style["texthalign"] = "center"
			style["textvalign"] = "bottom"
			apply( _draw_text, (device, (x,y), self.title), style )

	def compose( self, device, region ):
		if self.empty():
			raise BigglesError( "empty container" )
		exterior = region.copy()
		if self.title is not None:
			offset = _size_relative( self.title_offset, exterior )
			fontsize = _fontsize_relative( \
				self.title_style["fontsize"], exterior, device )
			exterior.deform( -offset-fontsize, 0, 0, 0 )
		interior = self.interior( device, exterior )
		self.compose_interior( device, interior )

	def page_compose( self, device ):
		device.open()
		bb = BoundingBox( device.lowerleft, device.upperright )
		device.bbox = bb.copy()
		for key,val in config.options('default').items():
			device.set( key, val )
		bb.expand( -self.page_margin )
		self.compose( device, bb )
		device.close()

	def show( self, width=None, height=None ):
		import os
		if width is None:
			width = config.value('screen','width')
		if height is None:
			height = config.value('screen','height')
		if os.name == 'posix':
			self.show_x11( width, height )
		elif os.name == 'dos' or os.name == 'nt':
			self.show_win( width, height )
		else:
			_message( "show: system type '%s' not supported" \
				% os.name )

	def show_x11( self, width, height ):
		persistent = config.interactive() and \
			config.bool('screen','persistent')
		device = renderer.ScreenRenderer( persistent, width, height )
		self.page_compose( device )
		device.delete()

	def show_win( self, width, height ):
		"""
		Substitute for show() that will work on Windows.
		Generates temporary files somewhere that end with
		'_biggles.png'. These temporary files are not deleted,
		they must be manually cleaned up during normal
		temp directory maintenance.
		"""
		import os, tempfile
		#tf = os.path.join( win_temp_path(), 'biggles_graph.png' )
		tf = tempfile.mktemp('_biggles.png')
		self.write_img( width, height, tf )
		os.startfile( tf )

	def psprint( self, printcmd=None, **kw ):
		import os, copy
		if os.name != 'posix':
			_message( "psprint: system type '%s' not supported" \
				% os.name )
		if printcmd is None:
			printcmd = config.value("printer","command")
		opt = copy.copy( config.options("postscript") )
		opt.update( kw )
		_message( 'printing plot with "%s"' % printcmd )
		printer = os.popen( printcmd, 'w' )
		device = apply( renderer.PSRenderer, (printer,), opt )
		self.page_compose( device )
		device.delete()
		printer.close()

 	def write_eps( self, filename, **kw ):
		opt = copy.copy( config.options("postscript") )
		opt.update( kw )
		file = _open_output( filename )
		device = apply( renderer.PSRenderer, (file,), opt )
		self.page_compose( device )
		device.delete()
		_close_output( file )

	def write_img( self, *args ):
		if len(args) == 4:
			type,width,height,filename = args
		elif len(args) == 3:
			import string
			width,height,filename = args
			type = string.lower( filename[-3:] )
		file = _open_output( filename )
		device = renderer.ImageRenderer( type, width, height, file )
		self.page_compose( device )
		device.delete()
		_close_output( file )

	save_as_eps = write_eps
	save_as_img = write_img

	def draw_piddle( self, canvastype=None, size=(500,500) ):
		from device.piddle import PiddleRenderer
		device = PiddleRenderer( canvastype, size )
		self.page_compose( device )
		canvas = device.canvas
		device.delete()
		return canvas

	def write_back_png( self, *args ):
		"""
		Saves PNG file in temporary file. Returns file contents.
		"""
		import tempfile, os
		if len(args) == 2:
			width,height = args
		type = 'png'
		file = tempfile.mktemp('_biggles.png')
		self.write_img(type, width, height, file)
		f = open(file, 'rb')
		output = f.read()
		f.close()
		os.remove(file)
		return output

def multipage( plots, filename, **kw ):
	file = _open_output( filename )
	opt = copy.copy( config.options("postscript") )
	opt.update( kw )
	device = apply( renderer.PSRenderer, (file,), opt )
	for plot in plots:
		plot.page_compose( device )
	device.delete()
	_close_output( file )

# -----------------------------------------------------------------------------

def _limits_axis( content_range, gutter, user_range, log ):

	r0, r1 = 0, 1

	if content_range is not None:
		a, b = content_range
		if a is not None: r0 = a
		if b is not None: r1 = b

	if gutter is not None:
		dx = 0.5 * gutter * (r1 - r0)
		a = r0 - dx
		if not log or a > 0:
			r0 = a
		r1 = r1 + dx

	if user_range is not None:
		a, b = user_range
		if a is not None: r0 = a
		if b is not None: r1 = b

	if r0 == r1:
		r0 = r0 - 1
		r1 = r1 + 1

	return r0, r1

def _limits( content_bbox, gutter, xlog, ylog, xrange, yrange ):

	xr = _limits_axis( content_bbox.xrange(), gutter, xrange, xlog )
	yr = _limits_axis( content_bbox.yrange(), gutter, yrange, ylog )

	return BoundingBox( (xr[0],yr[0]), (xr[1],yr[1]) )

# Plot ------------------------------------------------------------------------

class Plot( _PlotContainer ):

	def __init__( self, **kw ):
		apply( _PlotContainer.__init__, (self,) )
		apply( self.conf_setattr, ("Plot",), kw )
		self.content = _PlotComposite()

	def __iadd__( self, other ):
		self.add( other )

	def empty( self ):
		return self.content.empty()

	def add( self, *args ):
		apply( self.content.add, args )

	def limits( self ):
		return _limits( self.content.limits(), self.gutter, \
			self.xlog, self.ylog, self.xrange, self.yrange )

	def compose_interior( self, device, region, limits=None ):
		if limits is None:
			limits = self.limits()
		context = _PlotContext( device, region, limits,
			xlog=self.xlog, ylog=self.ylog )
		self.content.render( context )

	def compose( self, device, region, limits=None ):
		interior = self.interior( device, region )
		self.compose_interior( device, interior, limits )

# FramedPlot ------------------------------------------------------------------

class FramedPlot( _PlotContainer ):

	def __init__( self, **kw ):
		apply( _PlotContainer.__init__, (self,) )
		self.content1 = _PlotComposite()
		self.content2 = _PlotComposite()
		self.x1 = _HalfAxisX()
		self.x1.ticklabels_dir = -1
		self.y1 = _HalfAxisY()
		self.y1.ticklabels_dir = -1
		self.x2 = _HalfAxisX()
		self.x2.draw_ticklabels = None
		self.y2 = _HalfAxisY()
		self.y2.draw_ticklabels = None
		self.frame = _Alias( self.x1, self.x2, self.y1, self.y2 )
		self.frame1 = _Alias( self.x1, self.y1 )
		self.frame2 = _Alias( self.x2, self.y2 )
		self.x = _Alias( self.x1, self.x2 )
		self.y = _Alias( self.y1, self.y2 )
		apply( self.conf_setattr, ("FramedPlot",), kw )

	_attr_map = {
		"xlabel"	: ("x1", "label"),
		"ylabel"	: ("y1", "label"),
		"xlog"		: ("x1", "log"),
		"ylog"          : ("y1", "log"),
		"xrange"	: ("x1", "range"),
		"yrange"	: ("y1", "range"),
		"xtitle"	: ("x1", "label"),
		"ytitle"	: ("y1", "label"),
	}

	def __getattr__( self, name ):
		if self._attr_map.has_key( name ):
			xs = self._attr_map[ name ]
			obj = self
			for x in xs[:-1]:
				obj = getattr( obj, x )
			return getattr( obj, xs[-1] )
		else:
			return self.__dict__[name]

 	def __setattr__( self, name, value ):
		if self._attr_map.has_key( name ):
			xs = self._attr_map[ name ]
			obj = self
			for x in xs[:-1]:
				obj = getattr( obj, x )
			setattr( obj, xs[-1], value )
		else:
			self.__dict__[name] = value

	def empty( self ):
		return self.content1.empty() and self.content2.empty()

	def add( self, *args ):
		apply( self.content1.add, args )

	def add2( self, *args ):
		apply( self.content2.add, args )

	def _xy2log( self ):
		return _first_not_none(self.x2.log, self.x1.log), \
		       _first_not_none(self.y2.log, self.y1.log)

	def _limits1( self ):
		return _limits( self.content1.limits(),
			self.gutter, self.x1.log, self.y1.log, \
			self.x1.range, self.y1.range )

	def _context1( self, device, region ):
		return _PlotContext( device, region, self._limits1(),
			xlog=self.x1.log, ylog=self.y1.log )

	def _limits2( self ):
		limits = self.content2.limits()
		if self.content2.empty():
			limits = self.content1.limits()
		xlog, ylog = self._xy2log()
		xrange = _first_not_none( self.x2.range, self.x1.range )
		yrange = _first_not_none( self.y2.range, self.y1.range )
		return _limits( limits, self.gutter, xlog, ylog, \
			xrange, yrange )

	def _context2( self, device, region ):
		xlog, ylog = self._xy2log()
		return _PlotContext( device, region, self._limits2(),
			xlog, ylog )

	def exterior( self, device, region ):
		bbox = BoundingBox()

		context1 = self._context1( device, region )
		bbox.union( self.x1.bbox(context1) )
		bbox.union( self.y1.bbox(context1) )

		context2 = self._context2( device, region )
		bbox.union( self.x2.bbox(context2) )
		bbox.union( self.y2.bbox(context2) )

		return bbox

	def compose_interior( self, device, region ):
		_PlotContainer.compose_interior( self, device, region )

		context1 = self._context1( device, region )
		context2 = self._context2( device, region )

		self.content1.render( context1 )
		self.content2.render( context2 )

		self.y2.render( context2 )
		self.x2.render( context2 )
		self.y1.render( context1 )
		self.x1.render( context1 )

class OldCustomFramedPlot( FramedPlot ):

	def __init__( self, **kw ):
		apply( FramedPlot.__init__, (self,), kw ) 
		self.x = self.x1
		self.y = self.y1

# Table -----------------------------------------------------------------------

class _Grid:

	def __init__( self, nrows, ncols, bbox, cellpadding=0, cellspacing=0 ):
		self.nrows = nrows
		self.ncols = ncols

		w, h = bbox.width(), bbox.height()
		cp = _size_relative( cellpadding, bbox )
		cs = _size_relative( cellspacing, bbox )

		self.origin = pt_add( bbox.lowerleft(), (cp,cp) )
		self.step_x = (w + cs)/ncols
		self.step_y = (h + cs)/nrows
		self.cell_dimen = self.step_x - cs - 2*cp, \
			self.step_y - cs - 2*cp

	def cell( self, i, j ):
		ii = self.nrows-1 - i
		p = pt_add( self.origin, (j*self.step_x,ii*self.step_y) )
		q = pt_add( p, self.cell_dimen )
		return BoundingBox( p, q )

class Table( _PlotContainer ):

	def __init__( self, rows, cols, **kw ):
		apply( _PlotContainer.__init__, (self,) )
		apply( self.conf_setattr, ("Table",), kw )
		self.rows = rows
		self.cols = cols
		self.content = {}

	def __getitem__( self, key ):
		return self.content[key]

	def __setitem__( self, key, value ):
		self.content[key] = value

	def set( self, i, j, obj ):
		self.content[i,j] = obj

	def get( self, i, j ):
		return self.content.get( (i,j), None )

	def exterior( self, device, interior ):
		ext = interior.copy()

		if self.align_interiors:
			g = _Grid( self.rows, self.cols, interior, \
				self.cellpadding, self.cellspacing )

			for key,obj in self.content.items():
				subregion = apply( g.cell, key )
				ext.union( obj.exterior(device, subregion) )
		return ext

	def compose_interior( self, device, interior ):
		_PlotContainer.compose_interior( self, device, interior )

		g = _Grid( self.rows, self.cols, interior, \
			self.cellpadding, self.cellspacing )

		for key,obj in self.content.items():
			subregion = apply( g.cell, key )
			if self.align_interiors:
				obj.compose_interior( device, subregion )
			else:
				obj.compose( device, subregion )

# FramedArray -----------------------------------------------------------------
#
# Hideous, but it works...
#

def _frame_draw( obj, device, region, limits, labelticks=(0,1,1,0) ):
	frame = Frame( labelticks=labelticks )
	context = _PlotContext( device, region, limits,
		xlog=obj.xlog, ylog=obj.ylog )
	frame.render( context )

def _frame_bbox( obj, device, region, limits, labelticks=(0,1,1,0) ):
	frame = Frame( labelticks=labelticks )
	context = _PlotContext( device, region, limits,
		xlog=obj.xlog, ylog=obj.ylog )
	return frame.bbox( context )

def _range_union( a, b ):
	if a is None: return b
	if b is None: return a
	return min(a[0],b[0]), max(a[1],b[1])

class FramedArray( _PlotContainer ):

	def __init__( self, nrows, ncols, **kw ):
		apply( _PlotContainer.__init__, (self,) )
		self.nrows = nrows
		self.ncols = ncols
		self.content = {}
		for i in range(nrows):
			for j in range(ncols):
				self.content[i,j] = Plot()
		apply( self.conf_setattr, ("FramedArray",), kw )

	_attr_distribute = [
		'gutter',
		'xlog',
		'ylog',
		'xrange',
		'yrange',
	]

	_attr_deprecated = {
		'labeloffset'	: 'label_offset',
		'labelsize'	: 'label_size',
	}

	def __setattr__( self, name, value ):
		if name in self._attr_distribute:
			for obj in self.content.values():
				setattr( obj, name, value )
		else:
			_name = self._attr_deprecated.get( name, name )
			self.__dict__[_name] = value

	def __getitem__( self, key ):
		return self.content[key]

	def _limits( self, i, j ):
		if self.uniform_limits:
			return self._limits_uniform()
		else:
			return self._limits_nonuniform( i, j )

	def _limits_uniform( self ):
		limits = BoundingBox()
		for obj in self.content.values():
			limits.union( obj.limits() )
		return limits

	def _limits_nonuniform( self, i, j ):
		lx = None
		for k in range(self.nrows):
			l = self.content[k,j].limits()
			lx = _range_union( l.xrange(), lx )
		ly = None
		for k in range(self.ncols):
			l = self.content[i,k].limits()
			ly = _range_union( l.yrange(), ly )
		return BoundingBox( (lx[0],ly[0]), (lx[1],ly[1]) )

	def _grid( self, interior ):
		return _Grid( self.nrows, self.ncols, interior,
			cellspacing=self.cellspacing )

	def _frames_bbox( self, device, interior ):
		bb = BoundingBox()
		g = self._grid( interior )
		corners = [(0,0),(self.nrows-1,self.ncols-1)]

		for key in corners:
			obj = self.content[key]
			subregion = apply( g.cell, key )
			limits = apply( self._limits, key )
			axislabels = [0,0,0,0]
			if key[0] == self.nrows-1:
				axislabels[1] = 1
			if key[1] == 0:
				axislabels[2] = 1
			bb.union( _frame_bbox(obj, device, subregion, \
					limits, axislabels) )

		return bb

	def exterior( self, device, interior ):
		bb = self._frames_bbox( device, interior )

		labeloffset = _size_relative( self.label_offset, interior )
		labelsize = _fontsize_relative( \
			self.label_size, interior, device )
		margin = labeloffset + labelsize

		if self.xlabel is not None:
			bb.deform( 0, margin, 0, 0 )
		if self.ylabel is not None:
			bb.deform( 0, 0, margin, 0 )

		return bb

	def _frames_draw( self, device, interior ):
		g = self._grid( interior )

		for key,obj in self.content.items():
			subregion = apply( g.cell, key )
			limits = apply( self._limits, key )
			axislabels = [0,0,0,0]
			if key[0] == self.nrows-1:
				axislabels[1] = 1
			if key[1] == 0:
				axislabels[2] = 1
			_frame_draw( obj, device, subregion, \
				 limits, axislabels )

	def _data_draw( self, device, interior ):
		g = self._grid( interior )

		for key,obj in self.content.items():
			subregion = apply( g.cell, key )
			limits = apply( self._limits, key )
			obj.compose_interior( device, subregion, limits )

	def _labels_draw( self, device, interior ):
		bb = self._frames_bbox( device, interior )

		labeloffset = _size_relative( self.label_offset, interior )
		labelsize = _fontsize_relative( \
			self.label_size, interior, device )

		device.save_state()
		device.set( 'fontsize', labelsize )
		device.set( 'texthalign', 'center' )
		if self.xlabel is not None:
			x = interior.center()[0]
			y = bb.yrange()[0] - labeloffset
			device.set( 'textvalign', 'top' )
			device.text( (x,y), self.xlabel )
		if self.ylabel is not None:
			x = bb.xrange()[0] - labeloffset
			y = interior.center()[1]
			device.set( 'textangle', 90. )
			device.set( 'textvalign', 'bottom' )
			device.text( (x,y), self.ylabel )
		device.restore_state()

	def add( self, *args ):
		for obj in self.content.values():
			apply( obj.add, args ) 

	def compose_interior( self, device, interior ):
		_PlotContainer.compose_interior( self, device, interior )
		self._data_draw( device, interior )
		self._frames_draw( device, interior )
		self._labels_draw( device, interior )

# Text ------------------------------------------------------------------------

class Text( _PlotContainer ):

	def __init__( self, text, **kw ):
		apply( _PlotContainer.__init__, (self,) )
		apply( self.conf_setattr, ("Text",), kw )
		import string
		self.lines = string.split( text, "\n" )
		if self.lines[0] == '':
			del self.lines[0]
		if self.lines[-1] == '':
			del self.lines[-1]

	def compose( self, device, region ):
		device.save_state()
		context = _PlotContext( device, region, region )
		fontsize = _fontsize_relative(
			self.fontsize, context.dev_bbox, device )
		device.set( 'fontsize', fontsize )

		block_w = 0
		for line in self.lines:
			block_w = max( block_w, device.textwidth(line) )
		dy = fontsize * self.lineheight
		block_h = fontsize + dy * (len(self.lines)-1)

		x0, y0 = region.center()
		y0 = y0 + block_h/2.

		if self.halign == 'left':
			x0 = x0 - (region.width() - block_w)/2.
		elif self.halign == 'right':
			x0 = x0 + (region.width() - block_w)/2.

		if self.valign == 'top':
			y0 = y0 + (region.height() - block_h)/2.
		elif self.valign == 'bottom':
			y0 = y0 - (region.height() - block_h)/2.

		if self.justify == 'left':
			x0 = x0 - block_w/2
		elif self.justify == 'right':
			x0 = x0 + block_w/2

		## render
		block = _PlotComposite()

		y = y0
		for line in self.lines:
			block.add( DataLabel(x0, y, line, \
				halign=self.justify, valign='top', \
				fontface=self.fontface, \
				fontsize=self.fontsize) )
			y = y - dy

		block.render( context )
		device.restore_state()

# Inset -----------------------------------------------------------------------

class _Inset:

	def __init__( self, p, q, plot ):
		self.plot_limits = BoundingBox( p, q )
		self.plot = plot

	def render( self, context ):
		region = self.bbox( context )
		self.plot.compose_interior( context.draw, region )

class DataInset( _Inset ):

	def bbox( self, context ):
		p = apply( context.geom, self.plot_limits.lowerleft() )
		q = apply( context.geom, self.plot_limits.upperright() )
		return BoundingBox( p, q )

	def limits( self ):
		return self.plot_limits.copy()

class PlotInset( _Inset ):

	def bbox( self, context ):
		p = apply( context.plot_geom, self.plot_limits.lowerleft() )
		q = apply( context.plot_geom, self.plot_limits.upperright() )
		return BoundingBox( p, q )

	def limits( self ):
		return BoundingBox()




## Automatically adapted for numpy.oldnumeric Apr 04, 2007 by alter_code1.py

#
# $Id: matlab.py,v 1.4 2001/11/16 04:19:21 mrnolta Exp $
#
# Copyright (C) 2001 :
#
#   Jamie Mazer <mazer@socrates.berkeley.edu>
#   Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

#
#  matlab-like quick plot tools
#
#	import biggles.matlab
#
#	plot(x,y,style)
#	hist(n, nbins)
#	errorbar(x,y,e,style)
#	hold_on()
#	hold_off()
#	clf()
#	drawnow() **required**
#	subplot(nr,nc,n) or subplot(nr,nc,r,c) (note: r & c zerobased)
#
#  NOTE: PLOTS ARE NOT DRAWN UNTIL YOU CALL DRAWNOW()!!
#

import biggles
import math
import numpy.oldnumeric as Numeric
import string

class _MatlabEmulation:

	LINES	= [ "solid", "longdashed", "dotted" ]

	SYMBOLS	= [ "filled circle", "plus", "asterisk", "circle",
		"cross", "square", "triangle", "diamond", "star",
		"inverted triangle", "octagon", "filled square",
		"filled triangle", "filled diamond",
		"filled inverted triangle" ]

	COLORS	= [ "black", "red", "green", "blue", "magenta", "cyan" ]

	LAST	= None
	HOLD	= None
	TABLE	= None
	NROWS	= 1
	NCOLS	= 1
	R	= 0
	C	= 0

	def parse_style( self, s ):
		l,c,d = None, 'black', None
		if string.find(s, '--') >= 0:
			l = 'longdashed'
			s = string.replace(s, '--', '')
		elif string.find(s, '-') >= 0:
			l = 'solid'
			s = string.replace(s, '-', '')
		elif string.find(s, ':') >= 0:
			l = 'dotted'
			s = string.replace(s, ':', '')
	
		if 'y' in s: c='yellow'
		elif 'm' in s: c='magenta'
		elif 'c' in s: c='cyan'
		elif 'r' in s: c='red'
		elif 'g' in s: c='green'
		elif 'b' in s: c='blue'
		elif 'k' in s: c='black'
		elif 'w' in s: c='white'
	
		if '.' in s: d = 'dot'
		elif 'o' in s: d = 'filled circle'
		elif 'x' in s: d = 'cross'
		elif '+' in s: d = 'plus'
		elif '*' in s: d = 'asterisk'
		elif 's' in s: d = 'filled square'
		elif 'd' in s: d = 'filled diamond'
		elif '^' in s: d = 'triangle'
		elif 'v' in s: d = 'inverted triangle'
		elif '<' in s: d = 'filled triangle'
		elif '>' in s: d = 'filled inverted triangle'
		elif 'p' in s: d = 'octagon'
		elif 'h' in s: d = 'filled octagon'
	
		return l,c,d

	def get_table( self ):
		if self.TABLE is None:
			self.TABLE = biggles.Table( self.NROWS, self.NCOLS )
		return self.TABLE

	def get_plot( self, p=None ):
		if p is None:
			if self.HOLD:
				p = self.HOLD
			else:
				t = self.get_table()
				p = biggles.FramedPlot()
				t[ self.R, self.C ] = p
	
		try:
			nth = p.mplotnum + 1
		except KeyError:
			nth = 0
		p.mplotnum = nth

		return p

	def get_style( self, p, style=None ):
		n = p.mplotnum
		if style:
			return self.parse_style( style )
		else:
			return self.LINES[n], self.COLORS[n], self.SYMBOLS[n]

_matlab = _MatlabEmulation()

def subplot( nr, nc, r, c=None ):
	global _matlab
	if _matlab.NROWS != nr or _matlab.NCOLS != nc:
        	_matlab.TABLE = None
	_matlab.NROWS = nr
	_matlab.NCOLS = nc
	if c is None:
		_matlab.R = int(float(r-1)/float(nc))
		_matlab.C = int(0.5+math.fmod(float(r)-1.0, float(nc)))
	else:
		_matlab.R = r
		_matlab.C = c

def hold_on():
	global _matlab
	_matlab.HOLD = _matlab.LAST

def hold_off():
	global _matlab
	_matlab.HOLD = None

def clf():
	global _matlab
	_matlab.LAST = None
	_matlab.HOLD = None
	_matlab.TABLE = None
	_matlab.NROWS = 1
	_matlab.NCOLS = 1
	_matlab.R = 0
	_matlab.C = 0

def drawnow( width=None, height=None ):
	global _matlab
	if _matlab.TABLE:
		_matlab.TABLE.show( width=width, height=height )

def xlabel( s ):
	global _matlab
	if _matlab.LAST:
		_matlab.LAST.xlabel = s
        
def ylabel( s ):
	global _matlab
	if _matlab.LAST:
		_matlab.LAST.ylabel = s

def title( s ):
	global _matlab
	if _matlab.LAST:
		_matlab.LAST.title = s

def plot( x=None, y=None, style=None, show=None, p=None ):
	global _matlab
	p = _matlab.get_plot( p )
	lstyle, color, mstyle = _matlab.get_style( p, style )

	if y is None:
		y = x
		x = range(len(y))
        
	if mstyle is not None:
		p.add(biggles.Points(x, y, color=color, symboltype=mstyle))
	if lstyle is not None:
		p.add(biggles.Curve(x, y, color=color, linetype=lstyle))

	if show:
		p.show()
	_matlab.LAST = p
	return p

def errorbar( x, y, e, style='ko-', show=None, p=None ):
	global _matlab
	p = _matlab.get_plot( p )
	lstyle, color, mstyle = _matlab.get_style( p, style )

	p.add( biggles.SymmetricErrorBarsY(x, y, e, color=color) )

	if mstyle is not None:
		p.add( biggles.Points(x, y, color=color, type=mstyle) )
	if lstyle is not None:
		p.add( biggles.Curve(x, y, color=color, type=lstyle) )

	if show:
		p.show()
	_matlab.LAST = p
	return p

def hist( v, nbins=10, vmin=None, vmax=None, show=None, p=None ):
	global _matlab
	p = _matlab.get_plot( p )
        
	if vmin is None:
		vmin = float(min(v))
	if vmax is None:
		vmax = float(max(v))
	binwidth = (vmax - vmin) / float(nbins-1)
	x = Numeric.arrayrange( vmin, vmax+binwidth, binwidth )
	y = Numeric.zeros( x.shape, 'i' )

	for i in range(len(v)):
		n = int(round((float(v[i]) - vmin) / binwidth, 0))
		try:
			y[n] = y[n] + 1
		except IndexError:
			pass

	xx = Numeric.zeros( 2*len(x) + 3, 'f' )
	yy = Numeric.zeros( 2*len(y) + 3 )
	xx[0] = x[0]
	yy[0] = 0

	for i in range(0, len(x)):
		xx[1+2*i],xx[1+2*i+1] = x[i],x[i]+binwidth
		yy[1+2*i],yy[1+2*i+1] = y[i],y[i]

	xx[1+2*i+2] = x[i]+binwidth
	yy[1+2*i+2] = 0;
	xx[1+2*i+3] = x[0];
	yy[1+2*i+3] = 0;

	p.add( biggles.FillBelow(xx, yy), biggles.Curve(xx, yy) )
	p.yrange = 0, max(y)

	if show is not None:
		p.show()
	_matlab.LAST = p
	return p




#
# $Id: renderer.py,v 1.17 2004/03/08 23:30:05 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import libplot, math
raw = libplot

from tex2libplot import tex2libplot

## polygon clipping

def sh_inside( p, dim, boundary, side ):
	return side*p[dim] >= side*boundary

def sh_intersection( s, p, dim, boundary ):
	mid = not dim
	g = 0.
	if p[dim] != s[dim]:
		g = (boundary - s[dim])/(p[dim] - s[dim])
	q = [0,0]
	q[dim] = boundary
	q[mid] = s[mid] + g*(p[mid] - s[mid])
	return q[0], q[1]

def sutherland_hodgman( polygon, dim, boundary, side ):
	out = []
	s = polygon[-1]
	s_inside = sh_inside( s, dim, boundary, side )
	for p in polygon:
		p_inside = sh_inside( p, dim, boundary, side )

		crosses = (p_inside and not s_inside) or \
		          (not p_inside and s_inside)
		if crosses:
			out.append( sh_intersection(s, p, dim, boundary) )

		if p_inside:
			out.append( p )

		s = p
		s_inside = p_inside

	return out

class RendererState:

	def __init__( self ):
		self.current = {}
		self.saved = []

	def set( self, name, value ):
		self.current[name] = value

	def get( self, name, notfound=None ):
		if self.current.has_key(name):
			return self.current[name]
		for i in range(len(self.saved)):
			d = self.saved[i]
			if d.has_key(name):
				return d[name]
		return notfound

	def save( self ):
		self.saved.insert( 0, self.current )
		self.current = {}

	def restore( self ):
		self.current = self.saved.pop(0)

def _hexcolor( hextriplet, scale=1 ):
	s = float(scale) / 0xff
	r = s * ((hextriplet >> 16) & 0xff) 
	g = s * ((hextriplet >>  8) & 0xff)
	b = s * ((hextriplet >>  0) & 0xff)
	return r, g, b

def _set_color( pl, color ):
	if type(color) == type(''):
		raw.set_colorname_fg( pl, color )
	else:
		r,g,b = _hexcolor( color )
		raw.set_color_fg( pl, r, g, b )

def _set_pen_color( pl, color ):
	if type(color) == type(''):
		raw.set_colorname_pen( pl, color )
	else:
		r,g,b = _hexcolor( color )
		raw.set_color_pen( pl, r, g, b )

def _set_fill_color( pl, color ):
	if type(color) == type(''):
		raw.set_colorname_fill( pl, color )
	else:
		r,g,b = _hexcolor( color )
		raw.set_color_fill( pl, r, g, b )

_pl_line_type = {
	"dot"		: "dotted",
	"dash"		: "shortdashed",
	"dashed"	: "shortdashed",
}

def _set_line_type( pl, type ):
	pl_type = _pl_line_type.get( type, type )
	raw.set_line_type( pl, pl_type )

class LibplotRenderer:

	def __init__( self, ll, ur, type='X', parameters=None, file=None ):
		self.lowerleft = ll
		self.upperright = ur
		self.pl = raw.new( type, parameters, file )

	def open( self ):
		self.state = RendererState()
		raw.begin_page( self.pl )
		apply( raw.space, \
			(self.pl,) + self.lowerleft + self.upperright )
		raw.clear( self.pl )

	def clear( self ):
		raw.clear( self.pl )

	def close( self ):
		if self.pl is not None:
			raw.end_page( self.pl )

	def delete( self ):
		if self.pl is not None:
			raw.delete( self.pl )
			self.pl = None

	def __del__( self ):
		self.delete()

	## state commands

	__pl_style_func = {
		"color"		: _set_color,
		"linecolor"	: _set_pen_color,
		"fillcolor"	: _set_fill_color,
		"linetype"	: _set_line_type,
		"linewidth"	: raw.set_line_size,
		"filltype"	: raw.set_fill_level,
		"fillmode"	: raw.set_fill_type,
		"fontface"	: raw.set_font_type,
		"fontsize"	: raw.set_font_size,
		"textangle"	: raw.set_string_angle,
	}

	def set( self, key, value ):
		self.state.set( key, value )
		if LibplotRenderer.__pl_style_func.has_key(key):
			method = LibplotRenderer.__pl_style_func[key]
			apply( method, (self.pl, value) )

	def get( self, parameter, notfound=None ):
		return self.state.get( parameter, notfound )

	def save_state( self ):
		self.state.save()
		raw.gsave( self.pl )

	def restore_state( self ):
		self.state.restore()
		raw.grestore( self.pl )

	## drawing commands

	def move( self, p ):
		raw.move( self.pl, p[0], p[1] )

	def lineto( self, p ):
		raw.lineto( self.pl, p[0], p[1] )

	def linetorel( self, p ):
		raw.linetorel( self.pl, p[0], p[1] )

	def line( self, p, q ):
		cr = self.get( "cliprect" )
		if cr is None:
			raw.line( self.pl, p[0], p[1], q[0], q[1] )
		else:
			raw.clipped_line( self.pl, \
				cr[0], cr[1], cr[2], cr[3], \
				p[0], p[1], q[0], q[1] )

	def rect( self, p, q ):
		raw.rect( self.pl, p[0], p[1], q[0], q[1] )

	def circle( self, p, r ):
		raw.circle( self.pl, p[0], p[1], r )

	def ellipse( self, p, rx, ry, angle=0. ):
		raw.ellipse( self.pl, p[0], p[1], rx, ry, angle )

	def arc( self, c, p, q ):
		raw.arc( self.pl, c[0], c[1], p[0], p[1], q[0], q[1] )

	__pl_symbol_type = {
		"none"				: 0,
		"dot"				: 1,
		"plus"				: 2,
		"asterisk"			: 3, 
		"circle"			: 4,
		"cross"				: 5,
		"square"			: 6,
		"triangle"			: 7,
		"diamond"			: 8,
		"star"				: 9, 
		"inverted triangle"		: 10, 
		"starburst"			: 11,
		"fancy plus"			: 12,
		"fancy cross"			: 13, 
		"fancy square"			: 14, 
		"fancy diamond"			: 15,
		"filled circle"			: 16,
		"filled square"			: 17,
		"filled triangle"		: 18, 
		"filled diamond"		: 19,
		"filled inverted triangle"	: 20,
		"filled fancy square"		: 21,
		"filled fancy diamond"		: 22,
		"half filled circle"		: 23,
		"half filled square"		: 24,
		"half filled triangle"		: 25,
		"half filled diamond"		: 26,
		"half filled inverted triangle" : 27,
		"half filled fancy square"	: 28,
		"half filled fancy diamond"	: 29,
		"octagon"			: 30,
		"filled octagon"		: 31,
	}

	def symbol( self, p ):
		self.symbols( [p[0]], [p[1]] )

	def symbols( self, x, y ):
		DEFAULT_SYMBOL_TYPE = "square"
		DEFAULT_SYMBOL_SIZE = 0.01
		type_str = self.state.get( "symboltype", DEFAULT_SYMBOL_TYPE )
		size = self.state.get( "symbolsize", DEFAULT_SYMBOL_SIZE )
		if len(type_str) == 1:
			type = ord(type_str[0])
		else:
			type = LibplotRenderer.__pl_symbol_type.get( type_str )

		cr = self.get( "cliprect" )
		if cr is None:
			raw.symbols( self.pl, x, y, type, size )
		else:
			raw.clipped_symbols( self.pl, x, y, type, size,
				cr[0], cr[1], cr[2], cr[3] )

	def colored_symbols( self, x, y, c ):
		DEFAULT_SYMBOL_TYPE = "square"
		DEFAULT_SYMBOL_SIZE = 0.01
		type_str = self.state.get( "symboltype", DEFAULT_SYMBOL_TYPE )
		size = self.state.get( "symbolsize", DEFAULT_SYMBOL_SIZE )
		if len(type_str) == 1:
			type = ord(type_str[0])
		else:
			type = LibplotRenderer.__pl_symbol_type.get( type_str )

		cr = self.get( "cliprect" )
		if cr is None:
			# This will cause an error: not written yet
			raw.colored_symbols( self.pl, x, y, type, size )
		else:
			raw.clipped_colored_symbols( self.pl, x, y, c, type, size,
				cr[0], cr[1], cr[2], cr[3] )

	def density_plot( self, densgrid, ((xmin,ymin), (xmax,ymax)) ):
		raw.density_plot( self.pl, densgrid,
				  xmin, xmax, ymin, ymax )

	def color_density_plot( self, densgrid, ((xmin,ymin), (xmax,ymax)) ):
		raw.color_density_plot( self.pl, densgrid,
					xmin, xmax, ymin, ymax )


	def curve( self, x, y ):
		cr = self.get( "cliprect" )
		if cr is None:
			raw.curve( self.pl, x, y )
		else:
			raw.clipped_curve( self.pl, x, y,
				cr[0], cr[1], cr[2], cr[3] )

	def polygon( self, points ):
		pts = points
		cr = self.get( "cliprect" )
		if cr is not None:
			pts = sutherland_hodgman( pts, 0, cr[0], +1 )
			pts = sutherland_hodgman( pts, 0, cr[1], -1 )
			pts = sutherland_hodgman( pts, 1, cr[2], +1 )
			pts = sutherland_hodgman( pts, 1, cr[3], -1 )
		self.move( pts[0] )
		map( self.lineto, pts[1:] )

	## text commands

	__pl_text_align = {
		"center"	: ord('c'),
		"baseline"	: ord('x'),
		"left"		: ord('l'),
		"right"		: ord('r'),
		"top"		: ord('t'),
		"bottom"	: ord('b'),
	}

	def text( self, p, str ):
		plstr = tex2libplot( str )
		hstr = self.state.get( "texthalign", "center" )
		vstr = self.state.get( "textvalign", "center" )
		hnum = LibplotRenderer.__pl_text_align.get( hstr )
		vnum = LibplotRenderer.__pl_text_align.get( vstr )
		raw.move( self.pl, p[0], p[1] )
		raw.string( self.pl, hnum, vnum, plstr )

	def textwidth( self, str ):
		plstr = tex2libplot( str )
		return raw.get_string_width( self.pl, plstr )

	def textheight( self, str ):
		return self.state.get( "fontsize" )	## XXX: kludge?

class NonInteractiveScreenRenderer( LibplotRenderer ):

	def __init__( self, width, height ):
		ll = 0, 0
		ur = width, height
		parameters = {
			"BITMAPSIZE": "%dx%d" % (width, height),
			"VANISH_ON_DELETE": "no",
		}
		LibplotRenderer.__init__( self, ll, ur, "X", parameters )

class InteractiveScreenRenderer( LibplotRenderer ):

	def __init__( self, width, height ):
		ll = 0, 0
		ur = width, height
		parameters = {
			"BITMAPSIZE": "%dx%d" % (width, height),
			"VANISH_ON_DELETE": "yes",
		}
		LibplotRenderer.__init__( self, ll, ur, "X", parameters )

	def close( self ):
		raw.flush( self.pl )

	def delete( self ):
		raw.flush( self.pl )

_saved_screen_renderer = None

def ScreenRenderer( persistent=0, width=512, height=512 ):

	if persistent == 1:
		global _saved_screen_renderer
		if _saved_screen_renderer is None:
			_saved_screen_renderer \
				= InteractiveScreenRenderer( width, height )
		_saved_screen_renderer.clear()
		return _saved_screen_renderer
	else:
		return NonInteractiveScreenRenderer( width, height )

def _str_size_to_pts( str ):
	import re
	m = re.compile(r"([\d.]+)([^\s]+)").match(str)
	num_xx = float(m.group(1))
	units = m.group(2)
	# convert to postscipt pt = in/72
	xx2pt = { "in":72, "pt":1, "mm":2.835, "cm":28.35 }
	num_pt = int(num_xx*xx2pt[units])
	return num_pt

class PSRenderer( LibplotRenderer ):

	def __init__( self, file, paper="", width="", height="", **kw ):
		ll = 0, 0
		ur = _str_size_to_pts(width), _str_size_to_pts(height)
		pagesize = "%s,xsize=%s,ysize=%s" % (paper,width,height)
		for key,val in kw.items():
			pagesize = pagesize +","+ key +"="+ val
		parameters = { "PAGESIZE": pagesize }
		LibplotRenderer.__init__( self, ll, ur, "ps", parameters, file )

class ImageRenderer( LibplotRenderer ):

	def __init__( self, type, width, height, file ):
		ll = 0, 0
		ur = width, height
		parameters = { "BITMAPSIZE": "%dx%d" % (width, height) }
		LibplotRenderer.__init__( self, ll, ur, type, parameters, file )




#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import Numeric, math

x = Numeric.arange( 1*math.pi, 3*math.pi, math.pi/30 )
c = Numeric.cos(x)
s = Numeric.sin(x)

p = biggles.FramedPlot()
p.aspect_ratio = 1
p.frame1.draw_grid = 1
p.frame1.tickdir = 1

p.x1.label = "bottom"
p.x1.subticks = 1

p.y1.label = "left"
p.y1.draw_spine = 0

p.x2.label = "top"
p.x2.range = 10, 1000
p.x2.log = 1

p.y2.label = "right"
p.y2.draw_ticks = 0
p.y2.ticklabels = [ "-1", "-1/2", "0", "1/2", "1" ]

p.add( biggles.Curve(x, c, type='dash') )
p.add( biggles.Curve(x, s) )

#p.write_img( 400, 400, "example6.png" )
#p.write_eps( "example6.eps" )
p.show()



#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import math, Numeric 

x = Numeric.arange( 0, 3*math.pi, math.pi/10 )
y = Numeric.sin(x)

p = biggles.FramedPlot()
p.title = "Title"
p.xlabel = "X axis"
p.ylabel = "Y axis"

p.add( biggles.Histogram(y) )
p.add( biggles.PlotLabel(.5, .5, "Histogram", color=0xcc0000) )

t1 = biggles.Table( 1, 2 )
t1[0,0] = p
t1[0,1] = p

t2 = biggles.Table( 2, 1 )
t2[0,0] = t1
t2[1,0] = p

#t2.write_img( 400, 400, "example3.png" )
#t2.write_eps( "example3.eps" )
t2.show()



## Automatically adapted for numpy.oldnumeric Apr 04, 2007 by alter_code1.py

#
# $Id: func.py,v 1.20 2002/01/24 22:07:10 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import biggles, config
import string

def plot( a, b=None, **kw ):
	if b is None:
		x = range(1,len(a)+1)
		y = a
	else:
		x = a
		y = b
	p = apply( biggles.FramedPlot, (), kw )
	p.add( biggles.Curve(x, y) )
	p.show()

def read_column( num, filename, atox=float, \
		comment_char=None, return_numpy=None ):
	import string
	if comment_char is None:
		comment_char = config.value('read_column','comment_char')
	if return_numpy is None:
		return_numpy = config.value('read_column','return_numpy')
	x = []
	f = open( filename )
	lines = map( string.strip, f.readlines() )
	f.close()
	lines = filter( None, lines )	# get rid of ''
	for line in lines:
		if line[0] == comment_char[0]:
			continue
		column = string.split( line )
		x.append( column[num] )
	x = map( atox, x )
	if return_numpy:
		import numpy.oldnumeric as Numeric
		return Numeric.array( x )
	return x

def read_rows( filename, atox=float, comment_char=None, return_numpy=None ):
	import string
	if comment_char is None:
		comment_char = config.value('read_rows','comment_char')
	if return_numpy is None:
		return_numpy = config.value('read_rows','return_numpy')
	if return_numpy:
		import numpy.oldnumeric as Numeric
	x = []
	f = open( filename )
	lines = map( string.strip, f.readlines() )
	f.close()
	lines = filter( None, lines )	# get rid of blank lines
	for line in lines:
		if line[0] == comment_char[0]:
			continue
		row = map( atox, string.split(line) )
		if return_numpy:
			row = Numeric.array( row )
		x.append( row )
	return x

def read_matrix( filename, atox=float, comment_char=None, return_numpy=None ):
	if comment_char is None:
		comment_char = config.value('read_matrix','comment_char')
	if return_numpy is None:
		return_numpy = config.value('read_matrix','return_numpy')
	m = read_rows( filename, atox=atox, comment_char=comment_char, \
		return_numpy=0 )
	if return_numpy:
		import numpy.oldnumeric as Numeric
		return Numeric.array( m )
	return m




#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import Numeric, math

x = Numeric.arange( 0, 3*math.pi, math.pi/30 )
c = Numeric.cos(x)
s = Numeric.sin(x)

p = biggles.FramedPlot()
p.title = "title"
p.xlabel = r"$x$"
p.ylabel = r"$\Theta$"

p.add( biggles.FillBetween(x, c, x, s) )
p.add( biggles.Curve(x, c, color="red") )
p.add( biggles.Curve(x, s, color="blue") )

#p.write_img( 400, 400, "example1.png" )
#p.write_eps( "example1.eps" )
p.show()



#
# $Id: tex2libplot.py,v 1.5 2002/08/18 22:04:07 mrnolta Exp $
#
# Copyright (C) 2000 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

#
# This is just a quick and dirty converter from simple TeX strings
# to libplot Hershey font strings. Basically just a lookup table.
#

import re, string

class TeXLexer:

	re_control_sequence = re.compile( r"^\\[a-zA-Z]+[ ]?|^\\[^a-zA-Z][ ]?" )

	def __init__( self, str ):
		self.str = str
		self.len = len(str)
		self.pos = 0
		self.token_stack = []

	def get_token( self ):
		if self.pos == self.len:
			return None

		if len(self.token_stack) > 0:
			return self.token_stack.pop()

		str = self.str[self.pos:]
		m = self.re_control_sequence.match(str)
		if m is not None:
			token = m.group()
			self.pos = self.pos + len(token)
			## consume trailing space
			if len(token) > 2 and token[-1] == ' ':
				token = token[:-1]
		else:
			token = str[0]
			self.pos = self.pos + 1

		return token

	def put_token( self, token ):
		self.token_stack.append( token )

	def peek( self ):
		token = self.get_token()
		self.put_token( token )
		return token

_common_token_dict = {
	r'\\'				: '\\',
	r'\$'				: r'$',
	r'\%'				: r'%',
	r'\#'				: r'#',
	r'\&'				: r'&',
#	r'\~'				: r'~',
	r'\{'				: r'{',
	r'\}'				: r'}',
	r'\_'				: r'_',
#	r'\^'				: r'^',

	r'~'				: r' ',
	r'\/'				: r'\r^',

	## special letters (p52)
#	r'\oe'				: r'',
#	r'\OE'				: r'',
	r'\ae'				: r'\ae',
	r'\AE'				: r'\AE',
	r'\aa'				: r'\oa',
	r'\AA'				: r'\oA',
	r'\o'				: r'\/o',
	r'\O'				: r'\/O',
#	r'\l'				: r'',
#	r'\L'				: r'',
	r'\ss'				: r'\ss',

	## ignore stray brackets
	r'{'				: r'',
	r'}'				: r'',
}

_text_token_dict = {
	## punctuation (p52)
	r'\`'				: r'\`',
	r"\'"				: r"\'",
	r'\^'				: r'\^',
	r'\"'				: r'\:',
	r'\~'				: r'\~',
	r'\c'				: r'\,',

	## non-math symbols (p438)
	r'\S'				: r'\sc',
	r'\P'				: r'\ps',
	r'\dag'				: r'\dg',
	r'\ddag'			: r'\dd',
}

_math_token_dict = {

	r'*'				: r'\**',

	## spacing
#	r' '				: r'',
	r'\ '				: r' ',
	r'\quad'			: r'\r1',	# 1 em
	r'\qquad'			: r'\r1\r1',	# 2 em
	r'\,'				: r'\r6',	# 3/18 em
#	r'\>'				: r'',		# 4/18 em
#	r'\;'				: r'',		# 5/18 em
	r'\!'				: r'\l6',	# -1/6 em

	## lowercase greek
	r'\alpha'			: r'\*a',
	r'\beta'			: r'\*b',
	r'\gamma'			: r'\*g',
	r'\delta'			: r'\*d',
	r'\epsilon'			: r'\*e',
#	r'\varepsilon'			: r'',
	r'\zeta'			: r'\*z',
	r'\eta'				: r'\*y',
	r'\theta'			: r'\*h',
	r'\vartheta'			: r'\+h', 
	r'\iota'			: r'\*i',
	r'\kappa'			: r'\*k',
	r'\lambda'			: r'\*l',
	r'\mu'				: r'\*m',
	r'\nu'				: r'\*n',
	r'\xi'				: r'\*c',
	r'\pi'				: r'\*p',
#	r'\varpi'			: r'',
	r'\rho'				: r'\*r',
#	r'\varrho'			: r'',
	r'\sigma'			: r'\*s',
	r'\varsigma'			: r'\ts',
	r'\tau'				: r'\*t',
	r'\upsilon'			: r'\*u',
	r'\phi'				: r'\*f',
	r'\varphi'			: r'\+f',
	r'\chi'				: r'\*x',
	r'\psi'				: r'\*q',
	r'\omega'			: r'\*w',

	## uppercase greek
	r'\Alpha'			: r'\*A',
	r'\Beta'			: r'\*B',
	r'\Gamma'			: r'\*G',
	r'\Delta'			: r'\*D',
	r'\Epsilon'			: r'\*E',
	r'\Zeta'			: r'\*Z',
	r'\Eta'				: r'\*Y',
	r'\Theta'			: r'\*H',
	r'\Iota'			: r'\*I',
	r'\Kappa'			: r'\*K',
	r'\Lambda'			: r'\*L',
	r'\Mu'				: r'\*M',
	r'\Nu'				: r'\*N',
	r'\Xi'				: r'\*C',
	r'\Pi'				: r'\*P',
	r'\Rho'				: r'\*R',
	r'\Sigma'			: r'\*S',
	r'\Tau'				: r'\*T',
	r'\Upsilon'			: r'\*U',
	r'\Phi'				: r'\*F',
	r'\Chi'				: r'\*X',
	r'\Psi'				: r'\*Q',
	r'\Omega'			: r'\*W',

	## miscellaneous
	r'\aleph'			: r'\Ah',
	r'\hbar'			: r'\hb',
	r'\ell'				: r'\#H0662',
	r'\wp'				: r'\wp',
	r'\Re'				: r'\Re',
	r'\Im'				: r'\Im',
	r'\partial'			: r'\pd',
	r'\infty'			: r'\if',
	r'\prime'			: r'\fm',
	r'\emptyset'			: r'\es',
	r'\nabla'			: r'\gr',
	r'\surd'			: r'\sr',
#	r'\top'				: r'',
#	r'\bot'				: r'',
	r'\|'				: r'\||',
	r'\angle'			: r'\/_',
#	r'\triangle'			: r'',
	r'\backslash'			: r'\\',
	r'\forall'			: r'\fa',
	r'\exists'			: r'\te',
	r'\neg'				: r'\no',
#	r'\flat'			: r'',
#	r'\natural'			: r'',
#	r'\sharp'			: r'',
	r'\clubsuit'			: r'\CL',
	r'\diamondsuit'			: r'\DI',
	r'\heartsuit'			: r'\HE',
	r'\spadesuit'			: r'\SP',

	## binary operations
	r'\pm'				: r'\+-',
	r'\mp'				: r'\-+',
#	r'\setminus'			: r'',
	r'\cdot'			: r'\md',
	r'\times'			: r'\mu',
	r'\ast'				: r'\**',
#	r'\star'			: r'',
#	r'\diamond'			: r'',
#	r'\circ'			: r'',
	r'\bullet'			: r'\bu',
	r'\div'				: r'\di',
	r'\cap'				: r'\ca',
	r'\cup'				: r'\cu',
#	r'\uplus'			: r'',
#	r'\sqcap'			: r'',
#	r'\sqcup'			: r'',
#	r'\triangleleft'		: r'',
#	r'\triangleright'		: r'',
#	r'\wr'				: r'',
#	r'\bigcirc'			: r'',
#	r'\bigtriangleup'		: r'',
#	r'\bigtriangledown'		: r'',
#	r'\vee'				: r'',
#	r'\wedge'			: r'',
	r'\oplus'			: r'\c+',
#	r'\ominus'			: r'',
	r'\otimes'			: r'\c*',
#	r'\oslash'			: r'',
#	r'\odot'			: r'',
	r'\dagger'			: r'\dg',
	r'\ddagger'			: r'\dd',
#	r'\amalg'			: r'',

	## relations
	r'\leq'				: r'\<=',
#	r'\prec'			: r'',
#	r'\preceq'			: r'',
	r'\ll'				: r'<<',
	r'\subset'			: r'\SB',
#	r'\subseteq'			: r'',
#	r'\sqsubseteq'			: r'',
	r'\in'				: r'\mo',
#	r'\vdash'			: r'',
#	r'\smile'			: r'',
#	r'\frown'			: r'',
	r'\geq'				: r'\>=',
#	r'\succ'			: r'',
#	r'\succeq'			: r'',
	r'\gg'				: r'>>',
	r'\supset'			: r'\SS',
#	r'\supseteq'			: r'',
#	r'\sqsupseteq'			: r'',
#	r'\ni'				: r'',
#	r'\dashv'			: r'',
	r'\mid'				: r'|',
	r'\parallel'			: r'\||',
	r'\equiv'			: r'\==',
	r'\sim'				: r'\ap',
	r'\simeq'			: r'\~-',
#	r'\asymp'			: r'',
	r'\approx'			: r'\~~',
	r'\cong'			: r'\=~',
#	r'\bowtie'			: r'',
	r'\propto'			: r'\pt',
#	r'\models'			: r'',
#	r'\doteq'			: r'',
	r'\perp'			: r'\pp',

	## arrows
	r'\leftarrow'			: r'\<-',
	r'\Leftarrow'			: r'\lA',
	r'\rightarrow'			: r'\->',
	r'\Rightarrow'			: r'\rA',
	r'\leftrightarrow'		: r'\<>',
	r'\Leftrightarrow'		: r'\hA',
#	r'\mapsto'			: r'',
#	r'\hookleftarrow'		: r'',
#	r'\leftharpoonup'		: r'',
#	r'\leftharpoondown'		: r'',
#	r'\rightleftharpoons'		: r'',
#	...
	r'\uparrow'			: r'\ua',
	r'\Uparrow'			: r'\uA',
	r'\downarrow'			: r'\da',
	r'\Downarrow'			: r'\dA',
#	r'\updownarrow'			: r'',
#	r'\Updownarrow'			: r'',
#	r'\nearrow'			: r'',
#	r'\searrow'			: r'',
#	r'\swarrow'			: r'',
#	r'\nwarrow'			: r'',

	## openings
	r'\lbrack'			: r'[',
	r'\lbrace'			: r'{',
	r'\langle'			: r'\la',
#	r'\lfloor'			: r'',
#	r'\lceil'			: r'',

	## closings
	r'\rbrack'			: r']',
	r'\rbrace'			: r'}',
	r'\rangle'			: r'\ra',
#	r'\rfloor'			: r'',
#	r'\rceil'			: r'',

	## alternate names
	r'\ne'				: r'\!=',
	r'\neq'				: r'\!=',
	r'\le'				: r'\<=',
	r'\ge'				: r'\>=',
	r'\to'				: r'\->',
	r'\gets'			: r'\<-',
#	r'\owns'			: r'',
	r'\land'			: r'\AN',
	r'\lor'				: r'\OR',
	r'\lnot'			: r'\no',
	r'\vert'			: r'|',
	r'\Vert'			: r'\||',

	## extensions
	r'\degree'			: r'\de',
	r'\deg'				: r'\de',
	r'\degr'			: r'\de',
	r'\arcdeg'			: r'\de',
}

def map_text_token( token ):
	if _text_token_dict.has_key(token):
		return _text_token_dict[token]
	else:
		return _common_token_dict.get( token, token )

def map_math_token( token ):
	if _math_token_dict.has_key(token):
		return _math_token_dict[token]
	else:
		return _common_token_dict.get( token, token )

def math_group( lexer ):
	output = ''
	bracketmode = 0
	while 1:
		token = lexer.get_token()
		if token is None:
			break

		if token == '{':
			bracketmode = 1
		elif token == '}':
			break
		else:
			output = output + map_math_token( token )
			if not bracketmode:
				break
	return output

font_code = [ r'\f0', r'\f1', r'\f2', r'\f3' ]

def tex2libplot( str ):
	output = ''
	mathmode = 0
	font_stack = []
	font = 1

	lexer = TeXLexer( str )
	while 1:
		token = lexer.get_token()
		if token is None:
			break

		append = ''

		if token == '$':
			mathmode = not mathmode
		elif token == '{':
			font_stack.append( font )
		elif token == '}':
			old_font = font_stack.pop()
			if old_font != font:
				font = old_font
				append = font_code[font]
		elif token == r'\rm':
			font = 1
			append = font_code[font]
		elif token == r'\it':
			font = 2
			append = font_code[font]
		elif token == r'\bf':
			font = 3
			append = font_code[font]
		elif not mathmode:
			append = map_text_token( token )
		elif token == '_':
			append = r'\sb' + math_group(lexer) + r'\eb'
			if lexer.peek() == '^':
				append = r'\mk' + append + r'\rt'
		elif token == '^':
			append = r'\sp' + math_group(lexer) + r'\ep'
			if lexer.peek() == '_':
				append = r'\mk' + append + r'\rt'
		else:
			append = map_math_token( token )

		output = output + append

	return output



#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import math, Numeric 

x = Numeric.arange( 0, 2*math.pi, math.pi/20 )
s = Numeric.sin(x)
c = Numeric.cos(x)

inset = biggles.FramedPlot()
inset.title = "inset"
inset.frame.draw_ticks = 0

inset.add( biggles.Curve(x, s, type="dashed") )

p = biggles.FramedPlot()
p.aspect_ratio = 1.
p.frame.tickdir = +1
p.frame.draw_spine = 0

p.add( biggles.SymmetricErrorBarsY(x, s, [0.2]*len(x)) )
p.add( biggles.Points(x, s, color="red") )
p.add( biggles.Inset((.6,.6), (.95,.95), inset) )

#p.write_img( 400, 400, "example4.png" )
#p.write_eps( "example4.eps" )
p.show()



#!/usr/bin/env python

import sys
sys.path.insert(1,'..')
sys.ps1 = None

import Numeric
from biggles.matlab import *

def press_return():
	print "[press return]"
	sys.stdin.readline()

x = Numeric.arrayrange(-10,10);
y = x**2;
e = y/4

subplot(2,3,1)
plot(list(x), list(y), 'ko-')

subplot(2,3,2)
plot(x, y, 'ro')

subplot(2,3,3)
plot(x, y, 'go')

subplot(2,3,4)
plot(x, y, 'b-')

subplot(2,3,5)
plot([0, 1], [0, 1], 'o')

drawnow(width=500, height=200)
press_return()

drawnow()
press_return()
clf()

plot(x, y, '.')
drawnow()
press_return()

clf()
plot(x, y, 'r*-')
hold_on()
plot(x, -y, 'bv-')
errorbar(x, y/2, e, 'ko-')
title('foo')
xlabel('xbar')
ylabel('ybar')
hold_off()
drawnow()

press_return()



#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import Numeric, RandomArray

p = biggles.FramedPlot()
p.xrange = 0, 100
p.yrange = 0, 100
p.aspect_ratio = 1

x = Numeric.arange( 0, 100, 5 )
yA = RandomArray.normal( 40, 10, (len(x),) )
yB = x + RandomArray.normal( 0, 5, (len(x),) )

a = biggles.Points( x, yA, type="circle" )
a.label = "a points"

b = biggles.Points( x, yB )
b.label = "b points"
b.style( type="filled circle" )

l = biggles.Slope( 1, type="dotted" )
l.label = "slope"

k = biggles.PlotKey( .1, .9, [a,b,l] )

p.add( l, a, b, k )

#p.write_img( 400, 400, "example2.png" )
#p.write_eps( "example2.eps" )
p.show()



#!/usr/bin/env python

import sys
sys.path.insert(1,'..')

import biggles
import string

err_msg = """
This example needs the following data file:

	http://biggles.sourceforge.net/data/continents
"""
		
try:
	m = biggles.read_rows( "continents" )
except IOError:
	print err_msg
	sys.exit(-1)

p = biggles.HammerAitoffPlot()
p.ribs_l = 2

for i in range(len(m)/2):
	l = m[2*i]
	b = m[2*i+1]
	p.add( biggles.Curve(l, b) )

#p.write_img( 400, 400, "example7.png" )
#p.write_eps( "example7.eps" )
p.show()



## Automatically adapted for numpy.oldnumeric Apr 04, 2007 by alter_code1.py

#
# $Id: geometry.py,v 1.24 2003/05/12 22:13:11 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import math, numpy.oldnumeric as Numeric

# pt_* functions --------------------------------------------------------------

def pt_add( u, v ):
	return u[0] + v[0], u[1] + v[1]

def pt_sub( u, v ):
	return u[0] - v[0], u[1] - v[1]

def pt_mul( a, u ):
	return a * u[0], a * u[1]

def pt_rot( u, angle ):
	c, s = math.cos(angle), math.sin(angle)
	return c*u[0] - s*u[1], s*u[0] + c*u[1]

def pt_len( u ):
	return math.hypot( u[0], u[1] )

def pt_angle( u ):
	return math.atan2( u[1], u[0] )

def pt_unit( u ):
	r = pt_len(u)
	return u[0]/r, u[1]/r

def pt_min( a, b ):
	if a is None: return b
	if b is None: return a
	return min(a[0],b[0]), min(a[1],b[1])

def pt_max( a, b ):
	if a is None: return b
	if b is None: return a
	x = max( a[0], b[0] )
	if x is None:
		x = min( a[0], b[0] )
	y = max( a[1], b[1] )
	if y is None:
		y = min( a[1], b[1] )
	return x, y

# BoundingBox -----------------------------------------------------------------

class BoundingBox:

	def __init__( self, *args ):
		if len(args) > 0:
			self.p0 = reduce( pt_min, args )
			self.p1 = reduce( pt_max, args )
		else:
			self.p0 = None
			self.p1 = None

	def __str__( self ):
		return "(%s,%s)" % (str(self.p0), str(self.p1))

	def copy( self ):
		return BoundingBox( self.p0, self.p1 )

	def is_null( self ):
		return self.p0 is None or self.p1 is None

	def width( self ):
		if self.is_null():
			return None
		else:
			return abs( self.p0[0] - self.p1[0] )

	def height( self ):
		if self.is_null():
			return None
		else:
			return abs( self.p0[1] - self.p1[1] )

	def diagonal( self ):
		if self.is_null():
			return None
		else:
			return math.hypot( self.width(), self.height() )

	def aspect_ratio( self ):
		if self.is_null():
			return None
		else:
			return self.height()/self.width()

	def xrange( self ):
		if self.is_null():
			return None
		else:
			return self.p0[0], self.p1[0]

	def yrange( self ):
		if self.is_null():
			return None
		else:
			return self.p0[1], self.p1[1]

	def lowerleft( self ):
		if self.is_null():
			return None
		else:
			return self.p0

	def upperleft( self ):
		if self.is_null():
			return None
		else:
			return self.p0[0], self.p1[1]

	def upperright( self ):
		if self.is_null():
			return None
		else:
			return self.p1

	def lowerright( self ):
		if self.is_null():
			return None
		else:
			return self.p1[0], self.p0[1]

	def center( self ):
		x = self.xrange()
		y = self.yrange()
		return (x[0]+x[1])/2., (y[0]+y[1])/2.

	def union( self, other ):
		self.p0 = pt_min( self.p0, other.p0 )
		self.p1 = pt_max( self.p1, other.p1 )

	def deform( self, dt, db, dl, dr ):
		self.p0 = pt_sub( self.p0, (dl,db) )
		self.p1 = pt_add( self.p1, (dr,dt) )

	def shift( self, dp ):
		self.p0 = pt_add( self.p0, dp )
		self.p1 = pt_add( self.p1, dp )

	def expand( self, factor ):
		dp = pt_mul( factor/2., (self.width(), self.height()) )
		self.p0 = pt_sub( self.p0, dp )
		self.p1 = pt_add( self.p1, dp )

	def rotate( self, angle, p ):
		a = pt_add(pt_rot(pt_sub( self.lowerleft(), p), angle), p)
		b = pt_add(pt_rot(pt_sub(self.lowerright(), p), angle), p)
		c = pt_add(pt_rot(pt_sub( self.upperleft(), p), angle), p)
		d = pt_add(pt_rot(pt_sub(self.upperright(), p), angle), p)
		self.p0 = pt_min( a, pt_min( b, pt_min( c, d ) ) )
		self.p1 = pt_max( a, pt_max( b, pt_max( c, d ) ) )

	def make_aspect_ratio( self, ratio ):
		if ratio < self.aspect_ratio():
			dh = self.height() - ratio * self.width()
			self.p0 = self.p0[0], self.p0[1] + dh/2
			self.p1 = self.p1[0], self.p1[1] - dh/2
		else:
			dw = self.width() - self.height() / ratio
			self.p0 = self.p0[0] + dw/2, self.p0[1]
			self.p1 = self.p1[0] - dw/2, self.p1[1]

	def contains( self, q ):
		if self.p0[0] <= q[0] and \
		   q[0] <= self.p1[0] and \
		   self.p0[1] <= q[1] and \
		   q[1] <= self.p1[1]:
			return 1
		else:
			return 0

# AffineTransform -------------------------------------------------------------

def _matrix_multipy( A, B ):
	C00 = A[0][0] * B[0][0] + A[0][1] * B[1][0]
	C01 = A[0][0] * B[0][1] + A[0][1] * B[1][1]
	C10 = A[1][0] * B[0][0] + A[1][1] * B[1][0]
	C11 = A[1][0] * B[0][1] + A[1][1] * B[1][1]
	return (C00, C01), (C10, C11)

class AffineTransform:

	def __init__( self ):
		self.t = 0., 0.
		self.m = (1., 0.), (0., 1.)

	def __call__( self, x, y ):
		p = self.t[0] + self.m[0][0] * x + self.m[0][1] * y
		q = self.t[1] + self.m[1][0] * x + self.m[1][1] * y
		return p, q

	def call_vec( self, x, y ):
		x_ = Numeric.asarray( x )
		y_ = Numeric.asarray( y )
		p = self.t[0] + self.m[0][0] * x_ + self.m[0][1] * y_
		q = self.t[1] + self.m[1][0] * x_ + self.m[1][1] * y_
		return p, q

	def compose( self, other ):
		self.t = self( other.t[0], other.t[1] )
		self.m = _matrix_multiply( self.m, other.m )

class RectilinearMap( AffineTransform ):

	def __init__( self, src, dest ):
		AffineTransform.__init__( self )
		sx = dest.width() / src.width()
		sy = dest.height() / src.height()
		p, q = dest.lowerleft(), src.lowerleft()
		tx = p[0] - sx * q[0]
		ty = p[1] - sy * q[1]
		self.t = tx, ty
		self.m = ( sx, 0. ), ( 0., sy )




#!/usr/bin/env python
#
# $Id: setup.py,v 1.29 2004/03/08 23:40:25 mrnolta Exp $
#
# Copyright (C) 2001 :
#
#	Berthold Hollmann <bhoel@starship.python.net>
#	Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

#
# distutils setup file for biggles originally contributed
# by Berthold Hollmann.
#
# Modified by Raik Gruenberg:
# * added additional lib and include folders
# * converted everything to new numpy
#

from distutils.core import setup, Extension
from distutils.command.install_data import install_data
import distutils.sysconfig

import sys

prefix = distutils.sysconfig.PREFIX

## libplot
if sys.platform == "win32":
	libplot_include_dirs = []
	libplot_library_dirs = []
	libplot_libraries = ["plot.dll"]
else:
	libplot_include_dirs = [prefix+"/include",
				"/usr/local/include"]
	libplot_library_dirs = [prefix+"/lib",
				prefix+"/lib64",
				"/usr/X11R6/lib",
				"/usr/X11R6/lib64",
				"/usr/local/lib"]
	libplot_libraries = ["plot","Xaw","Xmu","Xt","SM","ICE","Xext","X11"]

# own install_data class to allow installation of data file
# (config.ini) to biggles directory

class my_install_data( install_data ):

	def finalize_options( self ):
		self.set_undefined_options( "install", \
			( "install_lib", "install_dir" ), \
			( "root", "root" ), \
			( "force", "force" ), \
		)

long_description = """\
Biggles is a Python module for creating publication-quality 2D scientific
plots. It supports multiple output formats (postscript, x11, png, svg, gif),
understands simple TeX, and sports a high-level, elegant interface. It's
intended for technical users with sophisticated plotting needs.
"""

setup(
	# Distribution meta-data

	name		= "python2-biggles",
	version		= "1.6.5.x",
	author		= "Mike Nolta",
	author_email	= "mrnolta@users.sourceforge.net",
	url		= "http://biggles.sourceforge.net/",
	license		= "GPL",
	description	= "scientific plotting module",
	long_description= long_description,

	# Description of the modules and packages in the distribution

	packages	= [ "biggles", "biggles.libplot" ],
	package_dir	= { "biggles" : "src" },

	ext_package	= "biggles",
	ext_modules	= [
		Extension( "_biggles", ["src/_biggles.c"] ),

		Extension( "libplot.libplot",
			["src/libplot/libplot.c"],
			include_dirs = libplot_include_dirs,
			library_dirs = libplot_library_dirs,
			libraries = libplot_libraries ), 
	],

	cmdclass	= { "install_data" : my_install_data },
	data_files	= [ ("biggles", ["src/config.ini"]) ]
)
